@@ -1,6 +1,6 @@
 /****************************************************************************
- * THIS module is based on Twm, but has been siginificantly modified 
- * by Rob Nation 
+ * This module is based on Twm, but has been siginificantly modified 
+ * by Rob Nation
  ****************************************************************************/
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
@@ -51,8 +51,6 @@
 #endif /* SHAPE */
 #include "module.h"
 
-char NoName[] = "Untitled"; /* name if no name is specified */
-
 /* Used to parse command line of clients for specific desk requests. */
 /* Todo: check for multiple desks. */
 static XrmDatabase db;
@@ -61,7 +59,7 @@ static XrmOptionDescRec table [] = {
    * to specify the desktop. I have to include dummy options that
    * are meaningless since Xrm seems to allow -w to match -workspace
    * if there would be no ambiguity. */
-    {"-workspacf",	"*junk",	XrmoptionSepArg, (caddr_t) NULL},
+    {"-workspacf",      "*junk",        XrmoptionSepArg, (caddr_t) NULL}, 
     {"-workspace",	"*desk",	XrmoptionSepArg, (caddr_t) NULL},
     {"-xrn",		NULL,		XrmoptionResArg, (caddr_t) NULL},
     {"-xrm",		NULL,		XrmoptionResArg, (caddr_t) NULL},
@@ -85,9 +83,7 @@ FvwmWindow *AddWindow(Window w)
   FvwmWindow *tmp_win;		        /* new fvwm window structure */
   unsigned long valuemask;		/* mask for create windows */
   XSetWindowAttributes attributes;	/* attributes for create windows */
-  Atom actual_type;
-  int actual_format,i,width,height;
-  unsigned long nitems, bytesafter;
+  int i,width,height;
   int a,b;
   char *value;
   unsigned long tflag;
@@ -99,8 +95,8 @@ FvwmWindow *AddWindow(Window w)
   char **client_argv = NULL, *str_type;
   Bool status;
   XrmValue rm_value;
-  unsigned long buttons;
   XTextProperty text_prop;
+  extern Bool PPosOverride;
 
   NeedToResizeToo = False;
   /* allocate space for the fvwm window */
@@ -114,24 +110,32 @@ FvwmWindow *AddWindow(Window w)
 
   tmp_win->cmap_windows = (Window *)NULL;
 
- if (XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
-		   &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth) == 0)
-    {
-      free((char *)tmp_win);
-      return(NULL);
-    }
-   if ( XGetWMName(dpy, tmp_win->w, &text_prop) != 0 ) 
-     tmp_win->name = (char *)text_prop.value ;
-   else
-     tmp_win->name = NoName;
+  if(!PPosOverride)
+    if (XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
+		     &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth) == 0)
+      {
+	free((char *)tmp_win);
+	return(NULL);
+      }
+  if ( XGetWMName(dpy, tmp_win->w, &text_prop) != 0 ) 
+    tmp_win->name = (char *)text_prop.value ;
+  else
+    tmp_win->name = NoName;
 
-  tmp_win->class = NoClass;
-  tmp_win->focus_sequence = 0;
+  /* removing NoClass change for now... */
+#if 0
+  tmp_win->class.res_name = tmp_win->class.res_class = NULL;
+#else
+  tmp_win->class.res_name = NoResource;
+  tmp_win->class.res_class = NoClass;
+#endif /* 0 */
   XGetClassHint(dpy, tmp_win->w, &tmp_win->class);
+#if 1
   if (tmp_win->class.res_name == NULL)
-    tmp_win->class.res_name = NoName;
+    tmp_win->class.res_name = NoResource;
   if (tmp_win->class.res_class == NULL)
-    tmp_win->class.res_class = NoName;
+    tmp_win->class.res_class = NoClass;
+#endif /* 1 */
 
   FetchWmProtocols (tmp_win);
   FetchWmColormapWindows (tmp_win);
@@ -174,30 +178,14 @@ FvwmWindow *AddWindow(Window w)
 
   tflag = LookInList(Scr.TheList,tmp_win->name,&tmp_win->class, &value, &Desk,
 		     &border_width, &resize_width,
-                     &forecolor,&backcolor,&tmp_win->buttons); 
+                     &forecolor,&backcolor,&tmp_win->buttons, 
+		     tmp_win->IconBox,&(tmp_win->BoxFillMethod));
 
   GetMwmHints(tmp_win);
 
   SelectDecor(tmp_win,tflag,border_width,resize_width); 
 
-  if(tflag & START_ICONIC_FLAG)
-    tmp_win->flags |= STARTICONIC;
-  if (tflag & STAYSONTOP_FLAG)
-    tmp_win->flags |= ONTOP;
-  if (tflag&STICKY_FLAG)
-    tmp_win->flags |= STICKY;
-  if(tflag & LISTSKIP_FLAG)
-    tmp_win->flags |= WINDOWLISTSKIP;
-  if(tflag & CIRCULATESKIP_FLAG)
-    tmp_win->flags |= CIRCULATESKIP;
-
-  if(tflag & SUPPRESSICON_FLAG)
-    tmp_win->flags |= SUPPRESSICON;
-  if(tflag & NOICON_TITLE_FLAG)
-    tmp_win->flags |= NOICON_TITLE;
-  if(Scr.flags & SuppressIcons)
-    tmp_win->flags |= SUPPRESSICON;
-
+  tmp_win->flags |= tflag & ALL_COMMON_FLAGS;
   /* find a suitable icon pixmap */
   if(tflag & ICON_FLAG)
     {
@@ -218,7 +206,6 @@ FvwmWindow *AddWindow(Window w)
 
   GetWindowSizeHints (tmp_win);
 
-
   /* Tentative size estimate */
   tmp_win->frame_width = tmp_win->attr.width+2*tmp_win->boundary_width;
   tmp_win->frame_height = tmp_win->attr.height + tmp_win->title_height+
@@ -232,8 +219,7 @@ FvwmWindow *AddWindow(Window w)
       status = XrmGetResource (db, "fvwm.desk", "Fvwm.Desk", &str_type, &rm_value);
       if ((status == True) && (rm_value.size != 0)) {
           Desk = atoi(rm_value.addr);
-	  tmp_win->flags |= STAYSONDESK_FLAG;
-	  tflag |= STAYSONDESK_FLAG;
+	  tflag |= STARTSONDESK_FLAG;
       }
       XrmDestroyDatabase (db);
       db = NULL;
@@ -260,9 +246,8 @@ FvwmWindow *AddWindow(Window w)
     }
 
   XSetWindowBorderWidth (dpy, tmp_win->w,0);
-  XGetWindowProperty (dpy, tmp_win->w, XA_WM_ICON_NAME, 0L, 200L, False,
-		      XA_STRING, &actual_type, &actual_format, &nitems,
-			  &bytesafter,(unsigned char **)&tmp_win->icon_name);
+  XGetWMIconName (dpy, tmp_win->w, &text_prop);
+  tmp_win->icon_name = (char *) text_prop.value;
   if(tmp_win->icon_name==(char *)NULL)
     tmp_win->icon_name = tmp_win->name;
 
@@ -271,18 +256,11 @@ FvwmWindow *AddWindow(Window w)
   tmp_win->flags &= ~MAXIMIZED;
 
 
-  tmp_win->TextPixel = Scr.StdColors.fore;
-  tmp_win->ReliefPixel = Scr.StdRelief.fore;
-  tmp_win->ShadowPixel = Scr.StdRelief.back;
-  tmp_win->BackPixel = Scr.StdColors.back;
+  tmp_win->TextPixel = Scr.MenuColors.fore;
+  tmp_win->ReliefPixel = Scr.MenuRelief.fore;
+  tmp_win->ShadowPixel = Scr.MenuRelief.back;
+  tmp_win->BackPixel = Scr.MenuColors.back;
 
-  if(tmp_win->flags & STICKY)
-    {
-      tmp_win->BackPixel = Scr.StickyColors.back;
-      tmp_win->ShadowPixel = Scr.StickyRelief.back;
-      tmp_win->ReliefPixel = Scr.StickyRelief.fore;
-      tmp_win->TextPixel = Scr.StickyColors.fore;
-    }
   if(forecolor != NULL)
     {
       XColor color;
@@ -322,7 +300,7 @@ FvwmWindow *AddWindow(Window w)
   tmp_win->frame_width = tmp_win->attr.width+2*tmp_win->boundary_width;
   tmp_win->frame_height = tmp_win->attr.height + tmp_win->title_height+
     2*tmp_win->boundary_width;
-
+  ConstrainSize(tmp_win, &tmp_win->frame_width, &tmp_win->frame_height);
 
   valuemask = CWBorderPixel | CWCursor | CWEventMask; 
   if(Scr.d_depth < 2)
@@ -344,11 +322,7 @@ FvwmWindow *AddWindow(Window w)
   attributes.event_mask = (SubstructureRedirectMask | ButtonPressMask | 
 			   ButtonReleaseMask |EnterWindowMask | 
 			   LeaveWindowMask |ExposureMask);
-  if(Scr.flags & SaveUnders)
-    {
-      valuemask |= CWSaveUnder;
-      attributes.save_under = TRUE;
-    }
+
   /* What the heck, we'll always reparent everything from now on! */
   tmp_win->frame = 
     XCreateWindow (dpy, Scr.Root, tmp_win->frame_x,tmp_win->frame_y, 
@@ -369,26 +343,6 @@ FvwmWindow *AddWindow(Window w)
 		    tmp_win->title_height),tmp_win->bw, CopyFromParent,
 		   InputOutput,CopyFromParent, valuemask,&attributes);  
 
-  if(Scr.flags & BackingStore)
-    {
-      valuemask |= CWBackingStore;
-      attributes.backing_store = WhenMapped;
-    }
-
-#ifndef NO_PAGER
-  if(Scr.Pager_w)
-    {
-      /* Create the pager_view window even if we're sticky, in case the
-       * user unsticks the window */
-      attributes.event_mask = ExposureMask;
-      tmp_win->pager_view= XCreateWindow (dpy, Scr.Pager_w, -10, -10, 2, 2, 1,
-					  CopyFromParent, InputOutput,
-					  CopyFromParent, valuemask,
-					  &attributes);
-      XMapRaised(dpy,tmp_win->pager_view);
-    }
-#endif
-  
   attributes.event_mask = (ButtonPressMask|ButtonReleaseMask|ExposureMask|
 			   EnterWindowMask|LeaveWindowMask);
   tmp_win->title_x = tmp_win->title_y = 0;
@@ -470,37 +424,20 @@ FvwmWindow *AddWindow(Window w)
   XReparentWindow(dpy, tmp_win->w, tmp_win->Parent,0,0);
 
   valuemask = (CWEventMask | CWDontPropagate);
-  attributes.event_mask = (StructureNotifyMask | PropertyChangeMask |
+  attributes.event_mask = (StructureNotifyMask | PropertyChangeMask | 
 			   VisibilityChangeMask |  EnterWindowMask | 
 			   LeaveWindowMask | 
 			   ColormapChangeMask | FocusChangeMask);
 
-#ifndef NO_PAGER
-  if(tmp_win->w == Scr.Pager_w)
-    {
-      Scr.FvwmPager = tmp_win;
-      attributes.event_mask |=ButtonPressMask|ButtonReleaseMask|ExposureMask
-	|ButtonMotionMask;
-      attributes.do_not_propagate_mask = ButtonPressMask;
-    }
-  else
-#endif
-    attributes.do_not_propagate_mask = ButtonPressMask | ButtonReleaseMask;
-  if(Scr.flags & AppsBackingStore)
-    {
-      valuemask |= CWBackingStore;
-      attributes.backing_store = WhenMapped;
-    }
+  attributes.do_not_propagate_mask = ButtonPressMask | ButtonReleaseMask;
+
   XChangeWindowAttributes (dpy, tmp_win->w, valuemask, &attributes);
   if ( XGetWMName(dpy, tmp_win->w, &text_prop) != 0 ) 
     tmp_win->name = (char *)text_prop.value ;
   else
     tmp_win->name = NoName;
   
-#ifndef NO_PAGER
-  if(tmp_win->w != Scr.Pager_w)
-#endif
-    XAddToSaveSet(dpy, tmp_win->w);
+  XAddToSaveSet(dpy, tmp_win->w);
 
   /*
    * Reparenting generates an UnmapNotify event, followed by a MapNotify.
@@ -553,7 +490,7 @@ FvwmWindow *AddWindow(Window w)
 			&a,&b,&JunkChild);
   tmp_win->xdiff -= a;
   tmp_win->ydiff -= b;
-  if(Scr.flags & ClickToFocus)
+  if(tmp_win->flags & ClickToFocus)
     {
       /* need to grab all buttons for window that we are about to
        * unhighlight */
@@ -574,6 +511,10 @@ FvwmWindow *AddWindow(Window w)
 		(unsigned long)tmp_win,tmp_win->name);
   BroadcastName(M_ICON_NAME,tmp_win->w,tmp_win->frame,
 		(unsigned long)tmp_win,tmp_win->icon_name);
+   if (tmp_win->icon_bitmap_file != NULL &&
+       tmp_win->icon_bitmap_file != Scr.DefaultIcon)
+     BroadcastName(M_ICON_FILE,tmp_win->w,tmp_win->frame,
+		   (unsigned long)tmp_win,tmp_win->icon_bitmap_file);
   BroadcastName(M_RES_CLASS,tmp_win->w,tmp_win->frame,
 		(unsigned long)tmp_win,tmp_win->class.res_class);
   BroadcastName(M_RES_NAME,tmp_win->w,tmp_win->frame,
@@ -589,7 +530,15 @@ FvwmWindow *AddWindow(Window w)
 		   Scr.MyDisplayHeight, 
 		   tmp_win->frame_x + (tmp_win->frame_width>>1), 
 		   tmp_win->frame_y + (tmp_win->frame_height>>1));
-      resize_window(tmp_win->w,tmp_win,0,0,0,0);
+      Event.xany.type = ButtonPress;
+      Event.xbutton.button = 1;
+      Event.xbutton.x_root = tmp_win->frame_x + (tmp_win->frame_width>>1);
+      Event.xbutton.y_root = tmp_win->frame_y + (tmp_win->frame_height>>1);
+      Event.xbutton.x = (tmp_win->frame_width>>1);
+      Event.xbutton.y = (tmp_win->frame_height>>1);
+      Event.xbutton.subwindow = None;
+      Event.xany.window = tmp_win->w;
+      resize_window(&Event , tmp_win->w, tmp_win, C_WINDOW, "", 0);
     }
   InstallWindowColormaps(colormap_win);
   return (tmp_win);
@@ -606,23 +555,24 @@ FvwmWindow *AddWindow(Window w)
  ***********************************************************************/
 void GrabButtons(FvwmWindow *tmp_win)
 {
-  MouseButton *MouseEntry;
+  Binding *MouseEntry;
 
-  MouseEntry = Scr.MouseButtonRoot;
-  while(MouseEntry != (MouseButton *)0)
+  MouseEntry = Scr.AllBindings;
+  while(MouseEntry != (Binding *)0)
     {
-      if((MouseEntry->func != (int)0)&&(MouseEntry->Context & C_WINDOW))
+      if((MouseEntry->Action != NULL)&&(MouseEntry->Context & C_WINDOW)
+	 &&(MouseEntry->IsMouse == 1))
 	{
-	  if(MouseEntry->Button >0)
+	  if(MouseEntry->Button_Key >0)
 	    {
-	      XGrabButton(dpy, MouseEntry->Button, MouseEntry->Modifier, 
+	      XGrabButton(dpy, MouseEntry->Button_Key, MouseEntry->Modifier, 
 			  tmp_win->w, 
 			  True, ButtonPressMask | ButtonReleaseMask,
 			  GrabModeAsync, GrabModeAsync, None, 
 			  Scr.FvwmCursors[DEFAULT]);
 	      if(MouseEntry->Modifier != AnyModifier)
 		{
-		  XGrabButton(dpy, MouseEntry->Button, 
+		  XGrabButton(dpy, MouseEntry->Button_Key, 
 			      (MouseEntry->Modifier | LockMask),
 			      tmp_win->w, 
 			      True, ButtonPressMask | ButtonReleaseMask,
@@ -670,7 +620,7 @@ void GrabButtons(FvwmWindow *tmp_win)
 		}
 	    }
 	}
-      MouseEntry = MouseEntry->NextButton;
+      MouseEntry = MouseEntry->NextBinding;
     }
   return;
 }
@@ -686,16 +636,17 @@ void GrabButtons(FvwmWindow *tmp_win)
  ***********************************************************************/
 void GrabKeys(FvwmWindow *tmp_win)
 {
-  FuncKey *tmp;
-  for (tmp = Scr.FuncKeyRoot.next; tmp != NULL; tmp = tmp->next)
+  Binding *tmp;
+  for (tmp = Scr.AllBindings; tmp != NULL; tmp = tmp->NextBinding)
     {
-      if(tmp->cont & (C_WINDOW|C_TITLE|C_RALL|C_LALL|C_SIDEBAR))
+      if((tmp->Context & (C_WINDOW|C_TITLE|C_RALL|C_LALL|C_SIDEBAR))&&
+	 (tmp->IsMouse == 0))
 	{
-	  XGrabKey(dpy, tmp->keycode, tmp->mods, tmp_win->frame, True,
+	  XGrabKey(dpy, tmp->Button_Key, tmp->Modifier, tmp_win->frame, True,
 		   GrabModeAsync, GrabModeAsync);
-	  if(tmp->mods != AnyModifier)
+	  if(tmp->Modifier != AnyModifier)
 	    {
-	      XGrabKey(dpy, tmp->keycode, tmp->mods|LockMask, 
+	      XGrabKey(dpy, tmp->Button_Key, tmp->Modifier|LockMask, 
 		       tmp_win->frame, True,
 		       GrabModeAsync, GrabModeAsync);	      
 	    }
@@ -822,7 +773,7 @@ void GetWindowSizeHints(FvwmWindow *tmp)
     tmp->hints.min_height = 1;
   if(tmp->hints.min_width <= 0)
     tmp->hints.min_width = 1;
-  
+
   if(!(tmp->hints.flags & PWinGravity))
     {
       tmp->hints.win_gravity = NorthWestGravity;
@@ -849,7 +800,7 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 unsigned long LookInList(name_list *list, char *name, XClassHint *class, 
 			 char **value, int *Desk, int *border_width,
 			 int *resize_width, char **forecolor, char **backcolor,
-                         unsigned long * buttons)
+                         unsigned long * buttons, int *IconBox, int *BoxFillMethod)
 {
   name_list *nptr;
   unsigned long retval = 0;
@@ -859,6 +810,14 @@ unsigned long LookInList(name_list *list, char *name, XClassHint *class,
   *backcolor = NULL;
   *Desk = 0;
   *buttons = 0;
+  *BoxFillMethod = 0;
+  *border_width = 0;
+  *resize_width = 0;
+  IconBox[0] = -1;
+  IconBox[1] = -1;
+  IconBox[2] = Scr.MyDisplayWidth;
+  IconBox[3] = Scr.MyDisplayHeight;
+
   /* look for the name first */
   for (nptr = list; nptr != NULL; nptr = nptr->next)
     {
@@ -868,7 +827,7 @@ unsigned long LookInList(name_list *list, char *name, XClassHint *class,
 	  if (matchWildcards(nptr->name,class->res_class) == TRUE)
 	    {
 	      if(nptr->value != NULL)*value = nptr->value;
-	      if(nptr->off_flags & STAYSONDESK_FLAG)
+	      if(nptr->off_flags & STARTSONDESK_FLAG)
 		*Desk = nptr->Desk;
 	      if(nptr->off_flags & BW_FLAG)
 		*border_width = nptr->border_width;
@@ -882,13 +841,22 @@ unsigned long LookInList(name_list *list, char *name, XClassHint *class,
 	      retval &= ~(nptr->on_flags);
               *buttons |= nptr->off_buttons;
               *buttons &= ~(nptr->on_buttons);
+	      if(nptr->BoxFillMethod != 0)
+		*BoxFillMethod = nptr->BoxFillMethod;
+	      if(nptr->IconBox[0] >= 0)
+		{
+		  IconBox[0] = nptr->IconBox[0];
+		  IconBox[1] = nptr->IconBox[1];
+		  IconBox[2] = nptr->IconBox[2];
+		  IconBox[3] = nptr->IconBox[3];
+		}
 	    }
 
 	  /* look for the res_name next */
 	  if (matchWildcards(nptr->name,class->res_name) == TRUE)
 	    {
 	      if(nptr->value != NULL)*value = nptr->value;
-	      if(nptr->off_flags & STAYSONDESK_FLAG)
+	      if(nptr->off_flags & STARTSONDESK_FLAG)
 		*Desk = nptr->Desk;
 	      if(nptr->off_flags & FORE_COLOR_FLAG)
 		*forecolor = nptr->ForeColor;
@@ -902,13 +870,22 @@ unsigned long LookInList(name_list *list, char *name, XClassHint *class,
 	      retval &= ~(nptr->on_flags);
               *buttons |= nptr->off_buttons;
               *buttons &= ~(nptr->on_buttons);
+	      if(nptr->BoxFillMethod != 0)
+		*BoxFillMethod = nptr->BoxFillMethod;
+	      if(nptr->IconBox[0] >= 0)
+		{
+		  IconBox[0] = nptr->IconBox[0];
+		  IconBox[1] = nptr->IconBox[1];
+		  IconBox[2] = nptr->IconBox[2];
+		  IconBox[3] = nptr->IconBox[3];
+		}
 	    }
 	}
       /* finally, look for name matches */
       if (matchWildcards(nptr->name,name) == TRUE)
 	{
 	  if(nptr->value != NULL)*value = nptr->value;
-	  if(nptr->off_flags & STAYSONDESK_FLAG)	   
+	  if(nptr->off_flags & STARTSONDESK_FLAG)	   
 	    *Desk = nptr->Desk;
 	  if(nptr->off_flags & FORE_COLOR_FLAG)
 	    *forecolor = nptr->ForeColor;
@@ -922,6 +899,15 @@ unsigned long LookInList(name_list *list, char *name, XClassHint *class,
 	  retval &= ~(nptr->on_flags);
           *buttons |= nptr->off_buttons;
           *buttons &= ~(nptr->on_buttons);
+	  if(nptr->BoxFillMethod != 0)
+	    *BoxFillMethod = nptr->BoxFillMethod;
+	  if(nptr->IconBox[0] >= 0)
+	    {
+	      IconBox[0] = nptr->IconBox[0];
+	      IconBox[1] = nptr->IconBox[1];
+	      IconBox[2] = nptr->IconBox[2];
+	      IconBox[3] = nptr->IconBox[3];
+	    }
 	}
     }
   return retval;
