@@ -65,10 +65,6 @@
 #endif /* SHAPE */
 #include "module.h"
 
-#ifndef NO_PAGER
-extern FvwmWindow *FindCounterpart(Window target);
-#endif /* NO_PAGER */
-
 unsigned int mods_used = (ShiftMask | ControlMask | Mod1Mask |
 			  Mod2Mask| Mod3Mask| Mod4Mask| Mod5Mask);
 extern int menuFromFrameOrWindowOrTitlebar;
@@ -90,6 +86,48 @@ void HandleShapeNotify(void);
 
 Window PressedW;
 
+/*
+** number of X events defined - it should be defined in X.h (to be
+** like 35), but just in case...
+*/
+#ifndef LASTEvent
+#define LASTEvent 256
+#endif /* !LASTEvent */
+typedef void (*PFEH)();
+PFEH EventHandlerJumpTable[LASTEvent];
+
+/*
+** Procedure:
+**   InitEventHandlerJumpTable
+*/
+void InitEventHandlerJumpTable(void)
+{
+  int i;
+
+  for (i=0; i<LASTEvent; i++)
+  {
+    EventHandlerJumpTable[i] = NULL;
+  }
+  EventHandlerJumpTable[Expose] =           HandleExpose;
+  EventHandlerJumpTable[DestroyNotify] =    HandleDestroyNotify;
+  EventHandlerJumpTable[MapRequest] =       HandleMapRequest;
+  EventHandlerJumpTable[MapNotify] =        HandleMapNotify;
+  EventHandlerJumpTable[UnmapNotify] =      HandleUnmapNotify;
+  EventHandlerJumpTable[ButtonPress] =      HandleButtonPress;
+  EventHandlerJumpTable[EnterNotify] =      HandleEnterNotify;
+  EventHandlerJumpTable[LeaveNotify] =      HandleLeaveNotify;
+  EventHandlerJumpTable[FocusIn] =          HandleFocusIn;
+  EventHandlerJumpTable[ConfigureRequest] = HandleConfigureRequest;
+  EventHandlerJumpTable[ClientMessage] =    HandleClientMessage;
+  EventHandlerJumpTable[PropertyNotify] =   HandlePropertyNotify;
+  EventHandlerJumpTable[KeyPress] =         HandleKeyPress;
+  EventHandlerJumpTable[VisibilityNotify] = HandleVisibilityNotify;
+  EventHandlerJumpTable[ColormapNotify] =   HandleColormapNotify;
+#ifdef SHAPE
+  EventHandlerJumpTable[ShapeEventBase+ShapeNotify] = HandleShapeNotify;
+#endif /* SHAPE */
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -100,6 +138,8 @@ void DispatchEvent()
 {
   Window w = Event.xany.window;
 
+  DBUG("DispatchEvent","Routine Entered");
+
   StashEventTime(&Event);
 
   if (XFindContext (dpy, w, FvwmContext, (caddr_t *) &Tmp_win) == XCNOENT)
@@ -107,67 +147,84 @@ void DispatchEvent()
   last_event_type = Event.type;
   last_event_window = w;
 
+#if 0
   switch(Event.type)
     {
     case Expose:
+      DBUG("DispatchEvent","Event = Expose");
       HandleExpose();
       break;
     case DestroyNotify:
+      DBUG("DispatchEvent","Event = DestroyNotify");
       HandleDestroyNotify();
       break;
     case MapRequest:
-      HandleMapRequest();
+      DBUG("DispatchEvent","Event = MapRequest");
+      HandleMapRequest(None);
       break;
     case MapNotify:
+      DBUG("DispatchEvent","Event = MapNotify");
       HandleMapNotify();
       break;
     case UnmapNotify:
+      DBUG("DispatchEvent","Event = UnmapNotify");
       HandleUnmapNotify();
       break;
-    case MotionNotify:
-      HandleMotionNotify();
-      break;
     case ButtonPress:
+      DBUG("DispatchEvent","Event = ButtonPress");
       HandleButtonPress();
       break;
-    case ButtonRelease:
-      HandleButtonRelease();
-      break;
     case EnterNotify:
+      DBUG("DispatchEvent","Event = EnterNotify");
       HandleEnterNotify();
       break;
     case LeaveNotify:
+      DBUG("DispatchEvent","Event = LeaveNotify");
       HandleLeaveNotify();
       break;
     case FocusIn:
+      DBUG("DispatchEvent","Event = FocusIn");
       HandleFocusIn();
       break;
     case ConfigureRequest:
+      DBUG("DispatchEvent","Event = ConfigureRequest");
       HandleConfigureRequest();
       break;
     case ClientMessage:
+      DBUG("DispatchEvent","Event = ClientMessage");
       HandleClientMessage();
       break;
     case PropertyNotify:
+      DBUG("DispatchEvent","Event = PropertyNotify");
       HandlePropertyNotify();
       break;
     case KeyPress:
+      DBUG("DispatchEvent","Event = KeyPress");
       HandleKeyPress();
       break;
     case VisibilityNotify:
+      DBUG("DispatchEvent","Event = VisibilityNotify");
       HandleVisibilityNotify();
       break;
     case ColormapNotify:
+      DBUG("DispatchEvent","Event = ColormapNotify");
       HandleColormapNotify();
       break;
     default:
 #ifdef SHAPE
-      if(Event.type == (ShapeEventBase + ShapeNotify)) 
+      if(Event.type == (ShapeEventBase + ShapeNotify))
+      {
+        DBUG("DispatchEvent","Event = ShapeNotify");
 	HandleShapeNotify();
+      }
 #endif /* SHAPE */
-      
       break;
     }
+#else
+  if (EventHandlerJumpTable[Event.type])
+    (*EventHandlerJumpTable[Event.type])();
+#endif /* 0 */
+  DBUG("DispatchEvent","Leaving Routine");
   return;
 }
 
@@ -180,6 +237,7 @@ void DispatchEvent()
  ************************************************************************/
 void HandleEvents()
 {
+  DBUG("HandleEvents","Routine Entered");
   while (TRUE)
     {
       last_event_type = 0;
@@ -288,14 +346,46 @@ void HandleFocusIn()
   
   if(!Tmp_win)
     {
-      SetBorder(Scr.Hilite,False,True,True,None);
-      Broadcast(M_FOCUS_CHANGE,3,0,0,0,0,0,0,0);
+      if(w != Scr.NoFocusWin)
+	{
+	  Scr.UnknownWinFocused = w;
+	}
+      else
+	{
+	  SetBorder(Scr.Hilite,False,True,True,None);
+	  Broadcast(M_FOCUS_CHANGE,5,0,0,0,Scr.HiColors.fore,
+		    Scr.HiColors.back,0,0);
+	  if (Scr.ColormapFocus == COLORMAP_FOLLOWS_FOCUS)
+	    {
+	      if((Scr.Hilite)&&(!(Scr.Hilite->flags & ICONIFIED)))
+		{
+		  InstallWindowColormaps(Scr.Hilite);
+		}
+	      else
+		{
+		  InstallWindowColormaps(NULL);
+		}
+	    }
+	  
+	}
     }
   else if(Tmp_win != Scr.Hilite)
     {
       SetBorder(Tmp_win,True,True,True,None);
-      Broadcast(M_FOCUS_CHANGE,3,Tmp_win->w,
-		Tmp_win->frame,(unsigned long)Tmp_win,0,0,0,0);
+      Broadcast(M_FOCUS_CHANGE,5,Tmp_win->w,
+		Tmp_win->frame,(unsigned long)Tmp_win,Scr.HiColors.fore,
+		Scr.HiColors.back,0,0);
+      if (Scr.ColormapFocus == COLORMAP_FOLLOWS_FOCUS)
+	{
+	  if((Scr.Hilite)&&(!(Scr.Hilite->flags & ICONIFIED)))
+	    {
+	      InstallWindowColormaps(Scr.Hilite);
+	    }
+	  else
+	    {
+	      InstallWindowColormaps(NULL);
+	    }
+	}
     }
 }
 
@@ -307,30 +397,30 @@ void HandleFocusIn()
  ************************************************************************/
 void HandleKeyPress()
 {
-  FuncKey *key;
+  Binding *key;
   unsigned int modifier;
-  Window dummy;
-  
-  Context = GetContext(Tmp_win,&Event,&dummy);
-
   modifier = (Event.xkey.state & mods_used);
-  for (key = Scr.FuncKeyRoot.next; key != NULL; key = key->next)
+  ButtonWindow = Tmp_win;
+ 
+  Context = GetContext(Tmp_win,&Event, &PressedW);
+  PressedW = None;
+
+  /* Here's a real hack - some systems have two keys with the
+   * same keysym and different keycodes. This converts all
+   * the cases to one keycode. */
+  Event.xkey.keycode = 
+    XKeysymToKeycode(dpy,XKeycodeToKeysym(dpy,Event.xkey.keycode,0));
+  
+  
+  for (key = Scr.AllBindings; key != NULL; key = key->NextBinding)
     {
-      ButtonWindow = Tmp_win;
-      /* Here's a real hack - some systems have two keys with the
-       * same keysym and different keycodes. This converts all
-       * the cases to one keycode. */
-      Event.xkey.keycode = 
-	XKeysymToKeycode(dpy,XKeycodeToKeysym(dpy,Event.xkey.keycode,0));
-      if ((key->keycode == Event.xkey.keycode) &&
-	  ((key->mods == (modifier&(~LockMask)))||
-	   (key->mods == AnyModifier)) &&
-	  (key->cont & Context))
+      if ((key->Button_Key == Event.xkey.keycode) &&
+	  ((key->Modifier == (modifier&(~LockMask)))||
+	   (key->Modifier == AnyModifier)) &&
+	  (key->Context & Context)&&
+	  (key->IsMouse == 0))
 	{
-	  ExecuteFunction(key->func, key->action, Event.xany.window,Tmp_win,
-			  &Event,Context,key->val1,key->val2,
-			  key->val1_unit,key->val2_unit,
-			  key->menu,-1);
+	  ExecuteFunction(key->Action,Tmp_win, &Event,Context,-1);
 	  return;
 	}
     }
@@ -346,6 +436,7 @@ void HandleKeyPress()
 	  XSendEvent(dpy, Tmp_win->w, False, KeyPressMask, &Event);
 	}
     }
+
   ButtonWindow = NULL;
 }
 
@@ -361,10 +452,7 @@ void HandleKeyPress()
 
 void HandlePropertyNotify()
 {
-  char *prop = NULL;
-  Atom actual = None;
-  int actual_format;
-  unsigned long nitems, bytesafter;
+  XTextProperty text_prop;
   
   if ((!Tmp_win)||(XGetGeometry(dpy, Tmp_win->w, &JunkRoot, &JunkX, &JunkY,
 				&JunkWidth, &JunkHeight, &JunkBW, &JunkDepth) == 0))
@@ -373,16 +461,14 @@ void HandlePropertyNotify()
   switch (Event.xproperty.atom) 
     {
     case XA_WM_NAME:
-      if (XGetWindowProperty (dpy, Tmp_win->w, Event.xproperty.atom, 0L, 
-			      MAX_NAME_LEN, False, XA_STRING, &actual,
-			      &actual_format, &nitems, &bytesafter,
-			      (unsigned char **) &prop) != Success ||
-	  actual == None)
+      if (!XGetWMName(dpy, Tmp_win->w, &text_prop))
 	return;
-      if (!prop) prop = NoName;
+
       free_window_names (Tmp_win, True, False);
       
-      Tmp_win->name = prop;
+      Tmp_win->name = (char *)text_prop.value;
+      if (Tmp_win->name == NULL)
+        Tmp_win->name = NoName;
       BroadcastName(M_WINDOW_NAME,Tmp_win->w,Tmp_win->frame,
 		    (unsigned long)Tmp_win,Tmp_win->name);
       
@@ -404,15 +490,12 @@ void HandlePropertyNotify()
       break;
       
     case XA_WM_ICON_NAME:
-      if (XGetWindowProperty (dpy, Tmp_win->w, Event.xproperty.atom, 0, 
-			      MAX_ICON_NAME_LEN, False, XA_STRING, &actual,
-			      &actual_format, &nitems, &bytesafter,
-			      (unsigned char **) &prop) != Success ||
-	  actual == None)
+      if (!XGetWMIconName (dpy, Tmp_win->w, &text_prop))
 	return;
-      if (!prop) prop = NoName;
       free_window_names (Tmp_win, False, True);
-      Tmp_win->icon_name = prop;
+      Tmp_win->icon_name = (char *) text_prop.value;
+      if (Tmp_win->icon_name == NULL)
+        Tmp_win->icon_name = NoName;
       BroadcastName(M_ICON_NAME,Tmp_win->w,Tmp_win->frame,
 		    (unsigned long)Tmp_win,Tmp_win->icon_name);
       RedoIconName(Tmp_win);
@@ -485,8 +568,20 @@ void HandlePropertyNotify()
       break;
       
     case XA_WM_NORMAL_HINTS:
-      GetWindowSizeHints (Tmp_win);
-      BroadcastConfig(M_CONFIGURE_WINDOW,Tmp_win);
+      {
+	int new_width, new_height;
+	
+	GetWindowSizeHints (Tmp_win);
+	new_width = Tmp_win->frame_width;
+	new_height = Tmp_win->frame_height;
+	ConstrainSize(Tmp_win, &new_width, &new_height);
+	if((new_width != Tmp_win->frame_width)||
+	   (new_height != Tmp_win->frame_height))
+	  SetupFrame(Tmp_win,Tmp_win->frame_x, Tmp_win->frame_y,
+		     new_width,new_height,False);
+
+	BroadcastConfig(M_CONFIGURE_WINDOW,Tmp_win);
+      }
       break;
       
     default:
@@ -499,12 +594,11 @@ void HandlePropertyNotify()
 	}
       else if(Event.xproperty.atom == _XA_WM_STATE)
 	{
-	  if((Scr.flags & ClickToFocus)&&(Tmp_win == Scr.Focus)&&
-	     (Tmp_win != NULL))
+	  if((Tmp_win != NULL)&&(Tmp_win->flags & ClickToFocus)
+	     &&(Tmp_win == Scr.Focus))
 	    {
 	      Scr.Focus = NULL;
-	      SetFocus(Tmp_win->w,Tmp_win);
-	      ClearCirculatedFlag();
+	      SetFocus(Tmp_win->w,Tmp_win,0);
 	    }
 	}
       break;
@@ -525,16 +619,31 @@ void HandleClientMessage()
   if ((Event.xclient.message_type == _XA_WM_CHANGE_STATE)&&
       (Tmp_win)&&(Event.xclient.data.l[0]==IconicState)&&
       !(Tmp_win->flags & ICONIFIED))
+  {
+    XQueryPointer( dpy, Scr.Root, &JunkRoot, &JunkChild,
+                   &(button.xmotion.x_root),
+                   &(button.xmotion.y_root),
+                   &JunkX, &JunkY, &JunkMask);
+    button.type = 0;
+    ExecuteFunction("Iconify",Tmp_win, &button,C_FRAME,-1);
+    return;
+  }
+
+  /*
+  ** CKH - if we get here, it was an unknown client message, so send
+  ** it to the client if it was in a window we know about.  I'm not so
+  ** sure this should be done or not, since every other window manager
+  ** I've looked at doesn't.  But it might be handy for a free drag and
+  ** drop setup being developed for Linux.
+  */
+  if (Tmp_win)
+  {
+    if(Event.xclient.window != Tmp_win->w)
     {
-      XQueryPointer( dpy, Scr.Root, &JunkRoot, &JunkChild,
-		    &(button.xmotion.x_root),
-		    &(button.xmotion.y_root),
-		    &JunkX, &JunkY, &JunkMask);
-      button.type = 0;
-      ExecuteFunction(F_ICONIFY, NULLSTR, Event.xany.window,
-		      Tmp_win, &button, C_FRAME,0,0, 0,0,
-		      (MenuRoot *)0,-1);
+      Event.xclient.window = Tmp_win->w;
+      XSendEvent(dpy, Tmp_win->w, False, NoEventMask, &Event);
     }
+  }
 }
 
 /***********************************************************************
@@ -550,13 +659,6 @@ void HandleExpose()
   
   if (Tmp_win)
     {
-#ifndef NO_PAGER
-      if((Tmp_win->w == Scr.Pager_w)||
-	 (Tmp_win->w == Scr.CPagerWin))
-	{
-	  ReallyRedrawPager();
-	}
-#endif
       if ((Event.xany.window == Tmp_win->title_w))
 	{
 	  SetTitleBar(Tmp_win,(Scr.Hilite == Tmp_win),False);
@@ -566,13 +668,6 @@ void HandleExpose()
 	  SetBorder(Tmp_win,(Scr.Hilite == Tmp_win),True,True,Event.xany.window);
 	}
     }
-#ifndef NO_PAGER
-  else
-    {
-      if(FindCounterpart(Event.xany.window))
-	ReallyRedrawPager();
-    }
-#endif
   return;
 }
 
@@ -599,8 +694,13 @@ void HandleDestroyNotify()
  *
  ************************************************************************/
 void HandleMapRequest()
+{
+  HandleMapRequestKeepRaised(None);
+}
+void HandleMapRequestKeepRaised(Window KeepRaised)
 {
   extern long isIconicState;
+  extern Bool PPosOverride;
   
   Event.xany.window = Event.xmaprequest.window;
   
@@ -608,7 +708,8 @@ void HandleMapRequest()
 		  (caddr_t *)&Tmp_win)==XCNOENT)
     Tmp_win = NULL;
   
-  XFlush(dpy);
+  if(!PPosOverride)
+    XFlush(dpy);
   
   /* If the window has never been mapped before ... */
   if(!Tmp_win)
@@ -619,6 +720,8 @@ void HandleMapRequest()
 	return;
     }
 
+  if(KeepRaised != None)
+    XRaiseWindow(dpy,KeepRaised);
   /* If it's not merely iconified, and we have hints, use them. */
   if (!(Tmp_win->flags & ICONIFIED))
     {
@@ -648,10 +751,10 @@ void HandleMapRequest()
 	      XMapWindow(dpy, Tmp_win->frame);
 	      Tmp_win->flags |= MAP_PENDING;
 	      SetMapStateProp(Tmp_win, NormalState);
-	      if(Scr.flags & ClickToFocus)
+	      if((Tmp_win->flags & ClickToFocus)&&
+		 ((!Scr.Focus)||(Scr.Focus->flags & ClickToFocus)))
 		{
-		  SetFocus(Tmp_win->w,Tmp_win);
-		  ClearCirculatedFlag();
+		  SetFocus(Tmp_win->w,Tmp_win,1);
 		}
 	    }
 	  else
@@ -665,7 +768,8 @@ void HandleMapRequest()
 	  Iconify(Tmp_win, 0, 0);
 	  break;
 	}
-      XSync(dpy,0);
+      if(!PPosOverride)
+	XSync(dpy,0);
       XUngrabServer(dpy);
     }
   /* If no hints, or currently an icon, just "deiconify" */
@@ -673,7 +777,8 @@ void HandleMapRequest()
     {
       DeIconify(Tmp_win);
     }
-  KeepOnTop();
+  if(!PPosOverride)
+    KeepOnTop();
 }
 
 
@@ -687,9 +792,20 @@ void HandleMapNotify()
 {
   if (!Tmp_win)
     {
+      if((Event.xmap.override_redirect == True)&&
+	 (Event.xmap.window != Scr.NoFocusWin))
+	{
+	  XSelectInput(dpy,Event.xmap.window,FocusChangeMask);
+	  Scr.UnknownWinFocused = Event.xmap.window;
+	}      
       return;
     }
 
+  /* Except for identifying over-ride redirect window mappings, we
+   * don't need or want windows associated with the sunstructurenotifymask */
+  if(Event.xmap.event != Event.xmap.window)
+    return;
+   
   /*
    * Need to do the grab to avoid race condition of having server send
    * MapNotify to client before the frame gets mapped; this is bad because
@@ -717,10 +833,10 @@ void HandleMapNotify()
 		(unsigned long)Tmp_win,0,0,0,0);
     }
   
-  if(Scr.flags & ClickToFocus)
+  if((Tmp_win->flags & ClickToFocus)&&(Scr.Focus)&&
+     ((!Scr.Focus)||(Scr.Focus->flags & ClickToFocus)))
     {
-      SetFocus(Tmp_win->w,Tmp_win);
-      ClearCirculatedFlag();
+      SetFocus(Tmp_win->w,Tmp_win,1);
     }
   if((!(Tmp_win->flags &(BORDER|TITLE)))&&(Tmp_win->boundary_width <2))
     {
@@ -750,6 +866,8 @@ void HandleUnmapNotify()
   XEvent dummy;
   extern FvwmWindow *colormap_win;
 
+  if(Event.xunmap.event != Event.xunmap.window)
+    return;
   /*
    * The July 27, 1988 ICCCM spec states that a client wishing to switch
    * to WithdrawnState should send a synthetic UnmapNotify with the
@@ -775,21 +893,22 @@ void HandleUnmapNotify()
   if(Scr.PreviousFocus == Tmp_win)
     Scr.PreviousFocus = NULL;
 
-  if((Tmp_win == Scr.Focus)&&(Scr.flags & ClickToFocus))
+  if((Tmp_win == Scr.Focus)&&(Tmp_win->flags & ClickToFocus))
     {
       if(Tmp_win->next)
-	SetFocus(Tmp_win->next->w,Tmp_win->next);
+	{
+	  HandleHardFocus(Tmp_win->next);
+	}
       else
-	SetFocus(Scr.NoFocusWin,NULL);
+	SetFocus(Scr.NoFocusWin,NULL,1);
     }
   
   if(Scr.Focus == Tmp_win)
-    SetFocus(Scr.NoFocusWin,NULL);
+    SetFocus(Scr.NoFocusWin,NULL,1);
 
   if(Tmp_win == Scr.pushed_window)
     Scr.pushed_window = NULL;
 
-  ClearCirculatedFlag();
   if(Tmp_win == colormap_win)
     colormap_win = NULL;
 
@@ -855,30 +974,6 @@ void HandleUnmapNotify()
 }
 
 
-/***********************************************************************
- *
- *  Procedure:
- *	HandleMotionNotify - MotionNotify event handler
- *
- **********************************************************************/
-void HandleMotionNotify()
-{
-#ifndef NO_PAGER
-  extern Bool EnablePagerRedraw;
-  
-  /* here is the code for dragging the viewport around within the pager.*/
-  if((Tmp_win)&&(Tmp_win->w == Scr.Pager_w)&&
-     (!(Tmp_win->flags & ICONIFIED)))
-    {
-      if(Event.xmotion.state == Button3MotionMask)
-	{
-	  EnablePagerRedraw = False;	  
-	  SwitchPages(FALSE,FALSE);
-	}
-    }
-#endif
-}
-
 /***********************************************************************
  *
  *  Procedure:
@@ -887,40 +982,27 @@ void HandleMotionNotify()
  ***********************************************************************/
 void HandleButtonPress()
 {
-#ifndef NO_PAGER
-  extern Bool EnablePagerRedraw;
-#endif
   unsigned int modifier;
-  MouseButton *MouseEntry;
-  char *Action;
+  Binding *MouseEntry;
   Window x;
   int LocalContext;
   
   /* click to focus stuff goes here */
-  if((Scr.flags & ClickToFocus)&&(Tmp_win != Scr.Ungrabbed)&&
+  if((Tmp_win)&&(Tmp_win->flags & ClickToFocus)&&(Tmp_win != Scr.Ungrabbed)&&
      ((Event.xbutton.state&
        (ControlMask|Mod1Mask|Mod2Mask|Mod3Mask|Mod4Mask|Mod5Mask)) == 0))
     {
       if(Tmp_win)
 	{
-	  SetFocus(Tmp_win->w,Tmp_win);
-	  ClearCirculatedFlag();
-       	  if(Scr.AutoRaiseDelay > 0)
-	    {
-	      SetTimer(Scr.AutoRaiseDelay);
-	    }
-	  else 
-	    {
+	  SetFocus(Tmp_win->w,Tmp_win,1);
 #ifdef CLICKY_MODE_1
-	      if((Event.xany.window != Tmp_win->w)&&
-		 (Event.xbutton.subwindow != Tmp_win->w)&&
-		 (Event.xany.window != Tmp_win->Parent)&&
-		 (Event.xbutton.subwindow != Tmp_win->Parent))
+	  if((Event.xany.window != Tmp_win->w)&&
+	     (Event.xbutton.subwindow != Tmp_win->w)&&
+	     (Event.xany.window != Tmp_win->Parent)&&
+	     (Event.xbutton.subwindow != Tmp_win->Parent))
 #endif
-		{
-		  if(Scr.AutoRaiseDelay == 0)
-		    RaiseWindow(Tmp_win);
-		}
+	    {
+	      RaiseWindow(Tmp_win);
 	    }
 	  
 	  KeepOnTop();
@@ -930,7 +1012,11 @@ void HandleButtonPress()
 	  if(!(Tmp_win->flags & ICONIFIED))
 	    {
 	      XSync(dpy,0);
+#if 1 /* pass click event to just clicked to focus window */
 	      XAllowEvents(dpy,ReplayPointer,CurrentTime);
+#else /* don't pass click to just focused window */
+	      XAllowEvents(dpy,AsyncPointer,CurrentTime);
+#endif /* 0 */
 	      XSync(dpy,0);
 	      return;
 	    }
@@ -941,26 +1027,6 @@ void HandleButtonPress()
   XAllowEvents(dpy,ReplayPointer,CurrentTime);
   XSync(dpy,0);
   
-  /* here is the code for moving windows in the pager.*/
-#ifndef NO_PAGER
-  if((Tmp_win)&&(Tmp_win->w == Event.xbutton.window)&&
-     (Tmp_win->w == Scr.Pager_w)&&(Event.xbutton.button == Button2))
-    {
-      PagerMoveWindow();
-      return;
-    }
-  if((Tmp_win)&&(Tmp_win->w == Scr.Pager_w)&&
-     (!(Tmp_win->flags & ICONIFIED))&&
-     (Event.xbutton.button == Button3)&&
-     (Event.xany.window == Scr.Pager_w))
-    {
-#ifndef NO_PAGER
-      EnablePagerRedraw = False;	  
-#endif
-      SwitchPages(FALSE,FALSE);
-    }
-#endif
-  
   Context = GetContext(Tmp_win,&Event, &PressedW);
   LocalContext = Context;
   x= PressedW;
@@ -976,27 +1042,20 @@ void HandleButtonPress()
   
   modifier = (Event.xbutton.state & mods_used);
   /* need to search for an appropriate mouse binding */
-  MouseEntry = Scr.MouseButtonRoot;
-  while(MouseEntry != (MouseButton *)0)
+  for (MouseEntry = Scr.AllBindings; MouseEntry != NULL;
+       MouseEntry= MouseEntry->NextBinding)
     {
-      if(((MouseEntry->Button == Event.xbutton.button)||(MouseEntry->Button == 0))&&
+      if(((MouseEntry->Button_Key == Event.xbutton.button)||
+	  (MouseEntry->Button_Key == 0))&&
 	 (MouseEntry->Context & Context)&&
 	 ((MouseEntry->Modifier == AnyModifier)||
-	  (MouseEntry->Modifier == (modifier& (~LockMask)))))
+	  (MouseEntry->Modifier == (modifier& (~LockMask))))&&
+	 (MouseEntry->IsMouse == 1))
 	{
 	  /* got a match, now process it */
-	  if (MouseEntry->func != (int)NULL)
-	    {
-	      Action = MouseEntry->item ? MouseEntry->item->action : NULL;
-	      ExecuteFunction(MouseEntry->func, Action, Event.xany.window, 
-			      Tmp_win, &Event, Context,MouseEntry->val1,
-			      MouseEntry->val2,
-			      MouseEntry->val1_unit,MouseEntry->val2_unit,
-			      MouseEntry->menu,-1);
-	      break;
-	    }
+	  ExecuteFunction(MouseEntry->Action,Tmp_win, &Event,Context,-1);
+	  break;
 	}
-      MouseEntry = MouseEntry->NextButton;
     }
   PressedW = None;
   if(LocalContext!=C_TITLE)
@@ -1006,36 +1065,6 @@ void HandleButtonPress()
   ButtonWindow = NULL;
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *	HandleButtonRelease - ButtonRelease event handler
- *
- ***********************************************************************/
-void HandleButtonRelease()
-{
-#ifndef NO_PAGER
-  extern Bool EnablePagerRedraw;
-  if((Tmp_win)&&(Event.xany.window == Scr.Pager_w)&&
-     (!(Tmp_win->flags & ICONIFIED)))
-    {
-      switch(Event.xbutton.button)
-	{
-	default:
-	case Button1:
-	  SwitchPages(TRUE,TRUE);
-	  break;
-	case Button3:
-	  EnablePagerRedraw = True;
-	  SwitchPages(FALSE,FALSE);
-	  RedrawPager();
-	  break;
-	}
-    }
-#endif
-}
-
-
 /***********************************************************************
  *
  *  Procedure:
@@ -1074,12 +1103,15 @@ void HandleEnterNotify()
  
   if(Event.xany.window == Scr.Root)
     {
-      if((!(Scr.flags & ClickToFocus))&&(!(Scr.flags & SloppyFocus)))
+      if((Scr.Focus)&&(!(Scr.Focus->flags&ClickToFocus))&&
+	 (!(Scr.Focus->flags&SloppyFocus)))
 	{
-	  SetFocus(Scr.NoFocusWin,NULL);
-	  ClearCirculatedFlag();
+	  SetFocus(Scr.NoFocusWin,NULL,1);
+	}
+      if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
+	{
+	  InstallWindowColormaps(NULL);
 	}
-      InstallWindowColormaps(NULL);
       return;
     }
   
@@ -1087,22 +1119,22 @@ void HandleEnterNotify()
   if (!Tmp_win) 
     return;
   
-  if(!(Scr.flags & ClickToFocus))
+  if(!(Tmp_win->flags & ClickToFocus))
     {
       if(Scr.Focus != Tmp_win)
 	{
-	  if((Scr.AutoRaiseDelay > 0)&&(!(Tmp_win->flags & VISIBLE)))
-	    SetTimer(Scr.AutoRaiseDelay);
-	  SetFocus(Tmp_win->w,Tmp_win);
-	  ClearCirculatedFlag();
+	  SetFocus(Tmp_win->w,Tmp_win,1);
 	}
       else
-	SetFocus(Tmp_win->w,Tmp_win);
+	SetFocus(Tmp_win->w,Tmp_win,1);
     }
-  if((!(Tmp_win->flags & ICONIFIED))&&(Event.xany.window == Tmp_win->w))
-    InstallWindowColormaps(Tmp_win);
-  else
-    InstallWindowColormaps(NULL);
+  if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
+  {
+    if((!(Tmp_win->flags & ICONIFIED))&&(Event.xany.window == Tmp_win->w))
+      InstallWindowColormaps(Tmp_win);
+    else
+      InstallWindowColormaps(NULL);
+  }
   return;
 }
 
@@ -1127,7 +1159,7 @@ void HandleLeaveNotify()
 	    {
 	      if(Scr.Focus != NULL)
 		{
-		  SetFocus(Scr.NoFocusWin,NULL);
+		  SetFocus(Scr.NoFocusWin,NULL,1);
 		}
 	      if(Scr.Hilite != NULL)
 		SetBorder(Scr.Hilite,False,True,True,None);
@@ -1262,9 +1294,6 @@ void HandleConfigureRequest()
   SetupFrame (Tmp_win, x, y, width, height,FALSE);
   KeepOnTop();
   
-#ifndef NO_PAGER
-  RedrawPager();
-#endif
 }
 
 /***********************************************************************
@@ -1321,19 +1350,6 @@ void HandleVisibilityNotify()
 }
 
 
-/**************************************************************************
- * 
- * For auto-raising windows, this routine is called
- *
- *************************************************************************/
-volatile int alarmed;
-void enterAlarm(int nonsense)
-{
-  alarmed = True;
-  signal(SIGALRM, enterAlarm);
-}
-
-
 /***************************************************************************
  *
  * Waits for next X event, or for an auto-raise timeout.
@@ -1342,41 +1358,13 @@ void enterAlarm(int nonsense)
 int My_XNextEvent(Display *dpy, XEvent *event)
 {
   extern int fd_width, x_fd;
-  struct itimerval value;
   fd_set in_fdset,out_fdset;
-  Window child;
   Window targetWindow;
   int i,count;
   int retval;
 
-  /* Do this prior to the select() call, in case the timer already expired,
-   * in which case the select would never return. */
-  if(alarmed)
-    {
-      alarmed = False;
-      XQueryPointer(dpy,Scr.Root,&JunkRoot,&child,&JunkX,&JunkY,&JunkX, 
-		    &JunkY,&JunkMask );
-      if((Scr.Focus != NULL)&&(child == Scr.Focus->frame))
-	{
-	  if(!(Scr.Focus->flags & VISIBLE))
-	    {
-	      RaiseWindow(Scr.Focus);
-	      KeepOnTop();
-#ifndef NO_PAGER
-	      RedrawPager();
-#endif
-	    }
-	}
-      return 0;
-    }
-  
-#ifndef HAVE_GETITIMER
-  value.it_value.tv_usec = 0;
-  value.it_value.tv_sec = 0;
-#else
-  getitimer(ITIMER_REAL,&value);
-#endif
-  
+  DBUG("My_XNextEvent","Routine Entered");
+
   FD_ZERO(&in_fdset);
   FD_SET(x_fd,&in_fdset);
   FD_ZERO(&out_fdset);
@@ -1403,30 +1391,24 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   XFlush(dpy);
   if(XPending(dpy))
     {
+      DBUG("My_XNextEvent","taking care of queued up events & returning");
       XNextEvent(dpy,event);
       StashEventTime(event);
       return 1;
     }
 
+  DBUG("My_XNextEvent","no X events waiting - about to reap children");
   /* Zap all those zombies! */
   /* If we get to here, then there are no X events waiting to be processed.
    * Just take a moment to check for dead children. */
   ReapChildren();
 
+  DBUG("My_XNextEvent","waiting for module input/output");
   XFlush(dpy);
-  if((value.it_value.tv_usec != 0)||
-     (value.it_value.tv_sec != 0))
 #ifdef __hpux
-    retval=select(fd_width,(int *)&in_fdset, 0, 0, &value.it_value);
+  retval=select(fd_width,(int *)&in_fdset, (int *)&out_fdset,0, NULL);
 #else
-    retval=select(fd_width,&in_fdset, 0, 0, &value.it_value);
-#endif
-  else
-#ifdef __hpux
-    retval=select(fd_width,(int *)&in_fdset, (int *)&out_fdset,
-		  0, NULL);
-#else
-    retval=select(fd_width,&in_fdset, &out_fdset, 0, NULL);
+  retval=select(fd_width,&in_fdset, &out_fdset, 0, NULL);
 #endif
   
   /* Check for module input. */
@@ -1439,10 +1421,12 @@ int My_XNextEvent(Display *dpy, XEvent *event)
 	      if((count = 
 		  read(readPipes[i],&targetWindow, sizeof(Window))) >0)
 		{
+                  DBUG("My_XNextEvent","calling HandleModuleInput");
 		  HandleModuleInput(targetWindow,i);
 		}
 	      if(count <= 0)
 		{
+                  DBUG("My_XNextEvent","calling KillModule");
 		  KillModule(i,10);
 		}
 	    }
@@ -1451,10 +1435,12 @@ int My_XNextEvent(Display *dpy, XEvent *event)
 	{
 	  if((retval>0)&&(FD_ISSET(writePipes[i], &out_fdset)))
 	    {
+              DBUG("My_XNextEvent","calling FlushQueue");
 	      FlushQueue(i);
 	    }
 	}
     }
+  DBUG("My_XNextEvent","leaving My_XNextEvent");
   return 0;
 }
 
