@@ -107,7 +107,6 @@ it should not be an option, once it works right. HEDU 2/2/94 */
 
 typedef struct MyFont
 {
-  char *name;			/* name of the font */
   XFontStruct *font;		/* font structure */
   int height;			/* height of the font */
   int y;			/* Y coordinate to draw characters */
@@ -144,9 +143,6 @@ typedef struct FvwmWindow
     Window right_w[5];
     Window icon_w;		/* the icon window */
     Window icon_pixmap_w;	/* the icon window */
-#ifndef NO_PAGER
-    Window pager_view;
-#endif
 #ifdef SHAPE
     int wShaped;               /* is this a shaped window */
 #endif
@@ -179,7 +175,7 @@ typedef struct FvwmWindow
     XWMHints *wmhints;		/* WM hints */
     XClassHint class;
     int Desk;                   /* Tells which desktop this window is on */
-	int FocusDesk;				/* Where (if at all) was it focussed */
+    int FocusDesk;		/* Where (if at all) was it focussed */
     int DeIconifyDesk;          /* Desk to deiconify to, for StubbornIcons */
     Window transientfor;
 
@@ -196,46 +192,62 @@ typedef struct FvwmWindow
     int functions;
     Window *cmap_windows;       /* Colormap windows property */
     int number_cmap_windows;    /* Should generally be 0 */
-    int focus_sequence;
     Pixel ReliefPixel;
     Pixel ShadowPixel;
     Pixel TextPixel;
     Pixel BackPixel;
     unsigned long buttons;
-
+    int IconBox[4];
+    int BoxFillMethod;
 } FvwmWindow;
 
 /***************************************************************************
  * window flags definitions 
  ***************************************************************************/
-#define STICKY         1 /* Does window stick to glass? */
-#define ONTOP          2 /* does window stay on top */
-#define BORDER         4 /* Is this decorated with border*/
-#define TITLE          8 /* Is this decorated with title */
-#define MAPPED        16 /* is it mapped? */
-#define ICONIFIED     32 /* is it an icon now? */
-#define TRANSIENT     64 /* is it a transient window? */
-#define RAISED       128 /* if its a sticky window, does it need to be raised*/
-#define VISIBLE      256 /* is the window fully visible */
-#define ICON_OURS    512 /* is the icon window supplied by the app? */
-#define XPM_FLAG    1024 /* is the icon window an xpm? */
-#define PIXMAP_OURS 2048 /* is the icon pixmap ours to free? */
-#define SHAPED_ICON 4096 /* is the icon shaped? */
-#define MAXIMIZED   8192 /* is the window maximized? */
-#define DoesWmTakeFocus		16384
-#define DoesWmDeleteWindow	32768
+/* The first 13 items are mapped directly from the style structure's
+ * flag value, so they MUST correspond to the first 13 entries in misc.h */
+#define STARTICONIC             (1<<0)
+#define ONTOP                   (1<<1) /* does window stay on top */
+#define STICKY                  (1<<2) /* Does window stick to glass? */
+#define WINDOWLISTSKIP          (1<<3)
+#define SUPPRESSICON            (1<<4)
+#define NOICON_TITLE            (1<<5)
+#define Lenience                (1<<6)
+#define StickyIcon              (1<<7)
+#define CirculateSkipIcon       (1<<8)
+#define CirculateSkip           (1<<9)
+#define ClickToFocus            (1<<10)
+#define SloppyFocus             (1<<11)
+#define SHOW_ON_MAP    (1<<12) /* switch to desk when it gets mapped? */
+#define ALL_COMMON_FLAGS (STARTICONIC|ONTOP|STICKY|WINDOWLISTSKIP| \
+			  SUPPRESSICON|NOICON_TITLE|Lenience|StickyIcon| \
+			  CirculateSkipIcon|CirculateSkip|ClickToFocus| \
+			  SloppyFocus|SHOW_ON_MAP) 
+
+#define BORDER         (1<<13) /* Is this decorated with border*/
+#define TITLE          (1<<14) /* Is this decorated with title */
+#define MAPPED         (1<<15) /* is it mapped? */
+#define ICONIFIED      (1<<16) /* is it an icon now? */
+#define TRANSIENT      (1<<17) /* is it a transient window? */
+#define RAISED         (1<<18) /* if its a sticky window, needs raising? */
+#define VISIBLE        (1<<19) /* is the window fully visible */
+#define ICON_OURS      (1<<20) /* is the icon window supplied by the app? */
+#define PIXMAP_OURS    (1<<21)/* is the icon pixmap ours to free? */
+#define SHAPED_ICON    (1<<22)/* is the icon shaped? */
+#define MAXIMIZED      (1<<23)/* is the window maximized? */
+#define DoesWmTakeFocus		(1<<24)
+#define DoesWmDeleteWindow	(1<<25)
 /* has the icon been moved by the user? */
-#define ICON_MOVED              65536 
+#define ICON_MOVED              (1<<26)
 /* was the icon unmapped, even though the window is still iconified
  * (Transients) */
-#define ICON_UNMAPPED          131072 
-#define WINDOWLISTSKIP         262144
-#define SUPPRESSICON           524288
-#define CIRCULATESKIP         1048576
-#define STARTICONIC           2097152
+#define ICON_UNMAPPED           (1<<27) 
 /* Sent an XMapWindow, but didn't receive a MapNotify yet.*/
-#define MAP_PENDING           4194304
-#define NOICON_TITLE          8388608
+#define MAP_PENDING             (1<<28)
+#define HintOverride            (1<<29)
+#define MWMButtons              (1<<30)
+#define MWMBorders              (1<<31)
+
 
 /* flags to suppress/enable title bar buttons */
 #define BUTTON1     1
@@ -254,13 +266,17 @@ extern void Reborder(void);
 extern void SigDone(int);
 extern void Restart(int nonsense);
 extern void Done(int, char *);
+extern void BlackoutScreen(void);
+extern void UnBlackoutScreen(void);
+
+extern int master_pid;
 
 extern Display *dpy;
 
-extern XClassHint NoClass;
-
 extern XContext FvwmContext;
 
+extern Window BlackoutWin;
+
 extern Window JunkRoot, JunkChild;
 extern int JunkX, JunkY;
 extern unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
