@@ -1,6 +1,6 @@
 /****************************************************************************
  * This module is mostly all new
- * by Rob Nation
+ * by Rob Nation 
  * A little of it is borrowed from ctwm.
  * Copyright 1993 Robert Nation. No restrictions are placed on this code,
  * as long as the copyright notice is preserved
@@ -40,7 +40,6 @@
 
 void GrabIconButtons(FvwmWindow *, Window);
 void GrabIconKeys(FvwmWindow *, Window);
-MyFont *IconFont;
 
 /****************************************************************************
  *
@@ -54,7 +53,6 @@ void CreateIconWindow(FvwmWindow *tmp_win, int def_x, int def_y)
   XSetWindowAttributes attributes;	/* attributes for create windows */
 
   tmp_win->flags |= ICON_OURS;
-  tmp_win->flags &= ~XPM_FLAG;
   tmp_win->flags &= ~PIXMAP_OURS;
   tmp_win->flags &= ~SHAPED_ICON;
   tmp_win->icon_pixmap_w = None;
@@ -90,7 +88,8 @@ void CreateIconWindow(FvwmWindow *tmp_win, int def_x, int def_y)
   /* figure out the icon window size */
   if (!(tmp_win->flags & NOICON_TITLE)||(tmp_win->icon_p_height == 0))
     {
-      tmp_win->icon_t_width = XTextWidth(IconFont->font,tmp_win->icon_name, 
+      tmp_win->icon_t_width = XTextWidth(Scr.IconFont.font,
+					 tmp_win->icon_name, 
                                          strlen(tmp_win->icon_name));
       tmp_win->icon_w_height = ICON_HEIGHT;
     }
@@ -126,9 +125,9 @@ void CreateIconWindow(FvwmWindow *tmp_win, int def_x, int def_y)
   tmp_win->icon_y_loc = final_y;
 
   /* clip to fit on screen */
-  attributes.background_pixel = Scr.StdColors.back;
+  attributes.background_pixel = Scr.MenuColors.back;
   valuemask =  CWBorderPixel | CWCursor | CWEventMask | CWBackPixel;
-  attributes.border_pixel = Scr.StdColors.fore;
+  attributes.border_pixel = Scr.MenuColors.fore;
   attributes.cursor = Scr.FvwmCursors[DEFAULT];
   attributes.event_mask = (ButtonPressMask | ButtonReleaseMask | 
 			   VisibilityChangeMask |
@@ -229,34 +228,21 @@ void DrawIconWindow(FvwmWindow *Tmp_win)
     }
   else
     {
-      if((Tmp_win->flags & STICKY)&&(Scr.d_depth < 2))
+      if(Scr.d_depth < 2)
 	{
-	  Relief = Scr.StickyShadowGC;
-	  Shadow = Scr.StickyShadowGC;
-	}
-      else if(Scr.d_depth < 2)
-	{
-	  Relief = Scr.StdShadowGC;
-	  Shadow = Scr.StdShadowGC;
+	  Relief = Scr.MenuGC;
+	  Shadow = Scr.MenuGC;
 	}
       else
 	{
-	  if(Tmp_win->BackPixel != Scr.StdColors.back)
-	    {
-	      Globalgcv.foreground = Tmp_win->ReliefPixel;
-	      Globalgcm = GCForeground;
-	      XChangeGC(dpy,Scr.ScratchGC1,Globalgcm,&Globalgcv); 
-	      Relief = Scr.ScratchGC1;
-	      
-	      Globalgcv.foreground = Tmp_win->ShadowPixel;
-	      XChangeGC(dpy,Scr.ScratchGC2,Globalgcm,&Globalgcv); 
-	      Shadow = Scr.ScratchGC2;
-	    }
-	  else
-	    {
-	      Relief = Scr.StdReliefGC;
-	      Shadow = Scr.StdShadowGC;
-	    }
+	  Globalgcv.foreground = Tmp_win->ReliefPixel;
+	  Globalgcm = GCForeground;
+	  XChangeGC(dpy,Scr.ScratchGC1,Globalgcm,&Globalgcv); 
+	  Relief = Scr.ScratchGC1;
+	  
+	  Globalgcv.foreground = Tmp_win->ShadowPixel;
+	  XChangeGC(dpy,Scr.ScratchGC2,Globalgcm,&Globalgcv); 
+	  Shadow = Scr.ScratchGC2;
 	}
       /* resize the icon name window */
       if(Tmp_win->icon_w != None)
@@ -275,13 +261,14 @@ void DrawIconWindow(FvwmWindow *Tmp_win)
     XSetWindowBackground(dpy,Tmp_win->icon_w,BackColor);
 
   /* write the icon label */
-  NewFontAndColor(IconFont->font->fid,TextColor,BackColor);
+  NewFontAndColor(Scr.IconFont.font->fid,TextColor,BackColor);
 
   if(Tmp_win->icon_pixmap_w != None)
     XMoveWindow(dpy,Tmp_win->icon_pixmap_w,Tmp_win->icon_x_loc,
                 Tmp_win->icon_y_loc);
   if(Tmp_win->icon_w != None)
     {
+      Tmp_win->icon_w_height = ICON_HEIGHT;
       XMoveResizeWindow(dpy, Tmp_win->icon_w, Tmp_win->icon_xl_loc,
                         Tmp_win->icon_y_loc+Tmp_win->icon_p_height,
                         Tmp_win->icon_w_width,ICON_HEIGHT);
@@ -299,11 +286,11 @@ void DrawIconWindow(FvwmWindow *Tmp_win)
     {
       if(Tmp_win->iconDepth == Scr.d_depth)
 	{
-	  XCopyArea(dpy,Tmp_win->iconPixmap,Tmp_win->icon_pixmap_w,Scr.FontGC,
+	  XCopyArea(dpy,Tmp_win->iconPixmap,Tmp_win->icon_pixmap_w,Scr.ScratchGC3,
 		    0,0,Tmp_win->icon_p_width-4, Tmp_win->icon_p_height-4,2,2);
 	}
       else
-	XCopyPlane(dpy,Tmp_win->iconPixmap,Tmp_win->icon_pixmap_w,Scr.FontGC,0,
+	XCopyPlane(dpy,Tmp_win->iconPixmap,Tmp_win->icon_pixmap_w,Scr.ScratchGC3,0,
 		   0,Tmp_win->icon_p_width-4, Tmp_win->icon_p_height-4,2,2,1);  
     }      
 
@@ -313,8 +300,9 @@ void DrawIconWindow(FvwmWindow *Tmp_win)
       x = (Tmp_win->icon_w_width - Tmp_win->icon_t_width)/2;
       if(x<3)x=3;
       
-      XDrawString (dpy, Tmp_win->icon_w, Scr.FontGC, x,
-                   Tmp_win->icon_w_height-IconFont->height+IconFont->y-3,
+      XDrawString (dpy, Tmp_win->icon_w, Scr.ScratchGC3, x,
+                   Tmp_win->icon_w_height-Scr.IconFont.height+
+		   Scr.IconFont.y-3,
                    Tmp_win->icon_name, strlen(Tmp_win->icon_name));
       RelieveWindow(Tmp_win,Tmp_win->icon_w,0,0,Tmp_win->icon_w_width,      
                     ICON_HEIGHT,Relief,Shadow, FULL_HILITE);
@@ -329,13 +317,6 @@ void DrawIconWindow(FvwmWindow *Tmp_win)
  ************************************************************************/
 void RedoIconName(FvwmWindow *Tmp_win)
 {
-#ifndef NO_PAGER
-  if(Scr.PagerFont.height > 0)
-    {
-      XClearWindow(dpy,Tmp_win->pager_view);
-      RedrawPager();
-    }
-#endif
 
   if(Tmp_win->flags & SUPPRESSICON)
     return;
@@ -343,7 +324,7 @@ void RedoIconName(FvwmWindow *Tmp_win)
   if (Tmp_win->icon_w == (int)NULL)
     return;
   
-  Tmp_win->icon_t_width = XTextWidth(IconFont->font,Tmp_win->icon_name, 
+  Tmp_win->icon_t_width = XTextWidth(Scr.IconFont.font,Tmp_win->icon_name, 
 				     strlen(Tmp_win->icon_name));
   /* clear the icon window, and trigger a re-draw via an expose event */
   if (Tmp_win->flags & ICONIFIED)
@@ -362,19 +343,28 @@ void RedoIconName(FvwmWindow *Tmp_win)
  ************************************************************************/
 void AutoPlace(FvwmWindow *t)
 {
-  int test_x=0, test_y=0,tw,th,tx,ty,i,temp_h,temp_w;
+  int test_x=0, test_y=0,tw,th,tx,ty,temp_h,temp_w;
   int base_x, base_y;
   int width,height;
   FvwmWindow *test_window;
   Bool loc_ok;
   int real_x=10, real_y=10;
+  int new_x, new_y;
 
   /* New! Put icon in same page as the center of the window */
   /* Not a good idea for StickyIcons */
-  if((Scr.flags & StickyIcons)||(t->flags & STICKY))
+  if((t->flags & StickyIcon)||(t->flags & STICKY))
     {
       base_x = 0;
       base_y = 0;
+      /*Also, if its a stickyWindow, put it on the current page! */
+      new_x = t->frame_x % Scr.MyDisplayWidth;
+      new_y = t->frame_y % Scr.MyDisplayHeight;
+      if(new_x < 0)new_x += Scr.MyDisplayWidth;
+      if(new_y < 0)new_y += Scr.MyDisplayHeight;
+      SetupFrame(t,new_x,new_y,
+		 t->frame_width,t->frame_height,False);
+      t->Desk = Scr.CurrentDesk;
     }
   else
     {
@@ -398,37 +388,33 @@ void AutoPlace(FvwmWindow *t)
       t->icon_x_loc = t->wmhints->icon_x;
       t->icon_y_loc = t->wmhints->icon_y;
     }
-  else
+  else if(t->IconBox[0] >= 0)
     {
       width = t->icon_p_width;
       height = t->icon_w_height + t->icon_p_height;
       loc_ok = False;  
       
       /* check all boxes in order */
-      i=0;
-      while((i<Scr.NumBoxes)&&(!loc_ok))
-	{
-	  /* In each IconBox, start at the upper left, travel right, then
-	   * down */
-	  test_y = Scr.IconBoxes[i][1]+base_y;
-	  
-	  temp_h = height;
-	  temp_w = width;
-	  
-	  /* OK second try at this.
-	   * If the window is taller than the icon box, ignore the icon height
-	   * when figuring where to put it. Same goes for the width */
-	  /* This should permit reasonably graceful handling of big icons. */
-	  if(width >= (Scr.IconBoxes[i][2] - Scr.IconBoxes[i][0]))
+      /* In each IconBox, start at the upper left, travel right, then
+       * down */
+      test_y = t->IconBox[1]+base_y;
+      
+      temp_h = height;
+      temp_w = width;
+      
+      /* OK second try at this.
+       * If the window is taller than the icon box, ignore the icon height
+       * when figuring where to put it. Same goes for the width */
+      /* This should permit reasonably graceful handling of big icons. */
+      if(width >= (t->IconBox[2] - t->IconBox[0]))
 	    temp_w = 0;
-	  if(height >= (Scr.IconBoxes[i][3] - Scr.IconBoxes[i][1]))
+	  if(height >= (t->IconBox[3] - t->IconBox[1]))
 	    temp_h = 0;
 	  
-	  while(((test_y + temp_h) < (Scr.IconBoxes[i][3]+base_y))&&(!loc_ok))
+	  while(((test_y + temp_h) < (t->IconBox[3]+base_y))&&(!loc_ok))
 	    {
-
-	      test_x = Scr.IconBoxes[i][0]+base_x;
-	      while(((test_x + temp_w) < (Scr.IconBoxes[i][2]+base_x))&&
+	      test_x = t->IconBox[0]+base_x;
+	      while(((test_x + temp_w) < (t->IconBox[2]+base_x))&&
 		    (!loc_ok))
 		{
 		  real_x = test_x;
@@ -464,25 +450,6 @@ void AutoPlace(FvwmWindow *t)
 				  loc_ok = False;
 				}
 			    }
-			  if(Scr.flags & StubbornIconPlacement)
-			    {
-			      if(!(test_window->flags&ICONIFIED)&&
-				 (test_window!=t))
-				{
-				  tw=test_window->frame_width;
-				  th=test_window->frame_height;
-				  tx = test_window->frame_x;
-				  ty = test_window->frame_y;
-			  
-				  if((tx < (real_x+width+3))&&
-				     ((tx+tw+3)>real_x)&&
-				     (ty < (real_y+height+3))&&
-				     ((ty+th+3)>real_y))
-				    {
-				      loc_ok = False;
-				    }
-				}
-			    }
 			}
 		      test_window = test_window->next;
 		    }
@@ -490,28 +457,27 @@ void AutoPlace(FvwmWindow *t)
 		}
 	      test_y +=3;
 	    }
-	  i++;
-	}
       if(loc_ok == False)
 	return;
       t->icon_x_loc = real_x;
       t->icon_y_loc = real_y;
+      
+      if(t->icon_pixmap_w)
+	XMoveWindow(dpy,t->icon_pixmap_w,t->icon_x_loc, t->icon_y_loc);
+  
+      t->icon_w_width = t->icon_p_width;
+      t->icon_xl_loc = t->icon_x_loc;
+
+      if (t->icon_w != None)
+	XMoveResizeWindow(dpy, t->icon_w, t->icon_xl_loc,
+			  t->icon_y_loc+t->icon_p_height, 
+			  t->icon_w_width,ICON_HEIGHT);
+      Broadcast(M_ICON_LOCATION,7,t->w,t->frame,
+		(unsigned long)t,
+		t->icon_x_loc,t->icon_y_loc,
+		t->icon_w_width, t->icon_w_height+ t->icon_p_height);
     }
-
-  if(t->icon_pixmap_w)
-    XMoveWindow(dpy,t->icon_pixmap_w,t->icon_x_loc, t->icon_y_loc);
-
-  t->icon_w_width = t->icon_p_width;
-  t->icon_xl_loc = t->icon_x_loc;
-
-  if (t->icon_w != None)
-    XMoveResizeWindow(dpy, t->icon_w, t->icon_xl_loc,
-                      t->icon_y_loc+t->icon_p_height, 
-                      t->icon_w_width,ICON_HEIGHT);
-  Broadcast(M_ICON_LOCATION,7,t->w,t->frame,
-	    (unsigned long)t,
-	    t->icon_x_loc,t->icon_y_loc,
-	    t->icon_w_width, t->icon_w_height+ t->icon_p_height);
+  
 }
 
 /***********************************************************************
@@ -525,15 +491,16 @@ void AutoPlace(FvwmWindow *t)
  ***********************************************************************/
 void GrabIconButtons(FvwmWindow *tmp_win, Window w)
 {
-  MouseButton *MouseEntry;
+  Binding *MouseEntry;
 
-  MouseEntry = Scr.MouseButtonRoot;
-  while(MouseEntry != (MouseButton *)0)
+  MouseEntry = Scr.AllBindings;
+  while(MouseEntry != (Binding *)0)
     {
-      if((MouseEntry->func != (int)0)&&(MouseEntry->Context & C_ICON))
+      if((MouseEntry->Action != NULL)&&(MouseEntry->Context & C_ICON)&&
+	 (MouseEntry->IsMouse == 1))
 	{
-	  if(MouseEntry->Button >0)
-	    XGrabButton(dpy, MouseEntry->Button, MouseEntry->Modifier, w, 
+	  if(MouseEntry->Button_Key >0)
+	    XGrabButton(dpy, MouseEntry->Button_Key, MouseEntry->Modifier, w, 
 			True, ButtonPressMask | ButtonReleaseMask,
 			GrabModeAsync, GrabModeAsync, None, 
 			Scr.FvwmCursors[DEFAULT]);
@@ -554,7 +521,7 @@ void GrabIconButtons(FvwmWindow *tmp_win, Window w)
 	    }
 	}
       
-      MouseEntry = MouseEntry->NextButton;
+      MouseEntry = MouseEntry->NextBinding;
     }
   return;
 }
@@ -572,11 +539,11 @@ void GrabIconButtons(FvwmWindow *tmp_win, Window w)
  ***********************************************************************/
 void GrabIconKeys(FvwmWindow *tmp_win,Window w)
 {
-  FuncKey *tmp;
-  for (tmp = Scr.FuncKeyRoot.next; tmp != NULL; tmp = tmp->next)
+  Binding *tmp;
+  for (tmp = Scr.AllBindings; tmp != NULL; tmp = tmp->NextBinding)
     {
-      if (tmp->cont & C_ICON)
-	XGrabKey(dpy, tmp->keycode, tmp->mods, w, True,
+      if ((tmp->Context & C_ICON)&&(tmp->IsMouse == 0))
+	XGrabKey(dpy, tmp->Button_Key, tmp->Modifier, w, True,
 		 GrabModeAsync, GrabModeAsync);
     }
   return;
@@ -638,7 +605,6 @@ void GetXPMFile(FvwmWindow *tmp_win)
     { 
       tmp_win->icon_p_width = xpm_attributes.width;
       tmp_win->icon_p_height = xpm_attributes.height;
-      tmp_win->flags |= XPM_FLAG;
       tmp_win->flags |= PIXMAP_OURS;
       tmp_win->iconDepth = Scr.d_depth;
 #ifdef SHAPE
@@ -664,7 +630,7 @@ void GetIconWindow(FvwmWindow *tmp_win)
 		  (unsigned int *)&tmp_win->icon_p_height, 
 		  &JunkBW, &JunkDepth)==0)
     {
-      fprintf(stderr,"Help! Bad Icon Window!\n");
+      fvwm_err("Help! Bad Icon Window!",NULL,NULL,NULL);
     }
   tmp_win->icon_p_width += JunkBW<<1;
   tmp_win->icon_p_height += JunkBW<<1;
@@ -723,8 +689,11 @@ void GetIconBitmap(FvwmWindow *tmp_win)
 void DeIconify(FvwmWindow *tmp_win)
 {
   FvwmWindow *t,*tmp;
-  int new_x,new_y,w2,h2;
 
+  if(!tmp_win)
+    return;
+
+  RaiseWindow(tmp_win);
   /* now de-iconify transients */
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
     {
@@ -734,45 +703,7 @@ void DeIconify(FvwmWindow *tmp_win)
 	  t->flags |= MAPPED;
 	  if(Scr.Hilite == t)
 	    SetBorder (t, False,True,True,None);
-	  /* make sure that the window is on this screen */
-	  if((t->frame_x < 0)||(t->frame_y<0)||
-	     (t->frame_x >= Scr.MyDisplayWidth)||
-	     (t->frame_y >= Scr.MyDisplayHeight))
-	    
-	    {
-	      /* try to put at least half the window
-	       * in the current screen, if the current desktop
-	       * is the windows desktop */
-	      if(Scr.flags & StubbornIcons)
-		t->Desk = t->DeIconifyDesk;
-	      else
-		t->Desk = Scr.CurrentDesk;
 
-	      if(t->Desk == Scr.CurrentDesk)
-		{
-		  new_x = t->frame_x;
-		  new_y = t->frame_y;
-		  w2 = (t->frame_width>>1);
-		  h2 = (t->frame_height>>1);
-		  if(!(Scr.flags & StubbornIcons))
-		    {
-		      if (( new_x < -w2) || (new_x > (Scr.MyDisplayWidth-w2 )))
-			{
-			  new_x = new_x % Scr.MyDisplayWidth;
-			  if ( new_x < -w2 )
-			    new_x += Scr.MyDisplayWidth;
-			}
-		      if ((new_y < -h2) || (new_y > (Scr.MyDisplayHeight-h2 )))
-			{
-			  new_y = new_y % Scr.MyDisplayHeight;
-			  if ( new_y < -h2 )
-			    new_y += Scr.MyDisplayHeight;
-			}
-		    }
-		  SetupFrame(t,new_x,new_y,
-			     t->frame_width,t->frame_height,False);
-		}
-	    }
 	  XMapWindow(dpy, t->w);
 	  if(t->Desk == Scr.CurrentDesk)
 	    {
@@ -797,14 +728,12 @@ void DeIconify(FvwmWindow *tmp_win)
 	  Broadcast(M_DEICONIFY,3,t->w,t->frame,(unsigned long)t,0,0,0,0);
 	}
     }
-  RaiseWindow(tmp_win);
 
-  if((Scr.flags & StubbornIcons)||(Scr.flags & ClickToFocus))
+  if(tmp_win->flags & ClickToFocus)
     FocusOn(tmp_win,1);
 
   KeepOnTop();
 
-  MoveResizePagerView(tmp_win);
   return;
 }
 
@@ -820,14 +749,15 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
   XWindowAttributes winattrs;
   unsigned long eventMask;
 
+  if(!tmp_win)
+    return;
   XGetWindowAttributes(dpy, tmp_win->w, &winattrs);
   eventMask = winattrs.your_event_mask;
 
   if((tmp_win)&&(tmp_win == Scr.Hilite)&&
-     (Scr.flags & ClickToFocus)&&(tmp_win->next))
+     (tmp_win->flags & ClickToFocus)&&(tmp_win->next))
     {
-      SetFocus(tmp_win->next->w,tmp_win->next);
-      ClearCirculatedFlag();
+      SetFocus(tmp_win->next->w,tmp_win->next,1);
     }
 
 
@@ -865,8 +795,6 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
 			t->icon_w_height+t->icon_p_height);
 	      BroadcastConfig(M_CONFIGURE_WINDOW,t);
 	    }
-	  if(t!= tmp_win)
-	    MoveResizePagerView(t);
 	}
     } 
   if (tmp_win->icon_w == None)
@@ -894,24 +822,42 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
       if(tmp_win->icon_pixmap_w != None)
 	XMapWindow(dpy, tmp_win->icon_pixmap_w);
       KeepOnTop();
-      MoveResizePagerView(tmp_win);
     }
-  if((Scr.flags & ClickToFocus)||(Scr.flags & SloppyFocus))
+  if((tmp_win->flags & ClickToFocus)||(tmp_win->flags & SloppyFocus))
     {
-      if ((tmp_win)&&(tmp_win == Scr.Focus))
+      if (tmp_win == Scr.Focus)
 	{
 	  if(Scr.PreviousFocus == Scr.Focus)
 	    Scr.PreviousFocus = NULL;
-	  if((Scr.flags & ClickToFocus)&&(tmp_win->next))
-	    SetFocus(tmp_win->next->w, tmp_win->next);
+	  if((tmp_win->flags & ClickToFocus)&&(tmp_win->next))
+	    SetFocus(tmp_win->next->w, tmp_win->next,1);
 	  else
 	    {
-	      SetFocus(Scr.NoFocusWin, NULL);
+	      SetFocus(Scr.NoFocusWin, NULL,1);
 	    }
-	  ClearCirculatedFlag();
 	}
     }
   return;
 }
 
 
+
+/****************************************************************************
+ *
+ * This is used to tell applications which windows on the screen are
+ * top level appication windows, and which windows are the icon windows
+ * that go with them.
+ *
+ ****************************************************************************/
+void SetMapStateProp(FvwmWindow *tmp_win, int state)
+{
+  unsigned long data[2];		/* "suggested" by ICCCM version 1 */
+  
+  data[0] = (unsigned long) state;
+  data[1] = (unsigned long) tmp_win->icon_w;
+/*  data[2] = (unsigned long) tmp_win->icon_pixmap_w;*/
+  
+  XChangeProperty (dpy, tmp_win->w, _XA_WM_STATE, _XA_WM_STATE, 32, 
+		   PropModeReplace, (unsigned char *) data, 2);
+  return;
+}
