@@ -1,33 +1,10 @@
 /****************************************************************************
- * This module is based on Twm, but has been siginificantly modified 
+ * This module is all original code 
  * by Rob Nation 
+ * Copyright 1993, Robert Nation
+ *     You may use this code for any purpose, as long as the original
+ *     copyright remains in the source code and all documentation
  ****************************************************************************/
-/*****************************************************************************/
-/**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
-/**                          Salt Lake City, Utah                           **/
-/**  Portions Copyright 1989 by the Massachusetts Institute of Technology   **/
-/**                        Cambridge, Massachusetts                         **/
-/**                                                                         **/
-/**                           All Rights Reserved                           **/
-/**                                                                         **/
-/**    Permission to use, copy, modify, and distribute this software and    **/
-/**    its documentation  for  any  purpose  and  without  fee is hereby    **/
-/**    granted, provided that the above copyright notice appear  in  all    **/
-/**    copies and that both  that  copyright  notice  and  this  permis-    **/
-/**    sion  notice appear in supporting  documentation,  and  that  the    **/
-/**    names of Evans & Sutherland and M.I.T. not be used in advertising    **/
-/**    in publicity pertaining to distribution of the  software  without    **/
-/**    specific, written prior permission.                                  **/
-/**                                                                         **/
-/**    EVANS & SUTHERLAND AND M.I.T. DISCLAIM ALL WARRANTIES WITH REGARD    **/
-/**    TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES  OF  MERCHANT-    **/
-/**    ABILITY  AND  FITNESS,  IN  NO  EVENT SHALL EVANS & SUTHERLAND OR    **/
-/**    M.I.T. BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL  DAM-    **/
-/**    AGES OR  ANY DAMAGES WHATSOEVER  RESULTING FROM LOSS OF USE, DATA    **/
-/**    OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER    **/
-/**    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE    **/
-/**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
-/*****************************************************************************/
 
 
 /***********************************************************************
@@ -38,8 +15,10 @@
 #include "../configure.h"
 
 #include <stdio.h>
+#include <unistd.h>
 #include <signal.h>
 #include <string.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <X11/keysym.h>
 #include <sys/types.h>
@@ -66,22 +45,28 @@ int Stashed_X, Stashed_Y,MenuY=0;
 void DrawTrianglePattern(Window,GC,GC,GC,int,int,int,int);
 void DrawSeparator(Window, GC,GC,int, int,int,int,int);
 void DrawUnderline(Window w, GC gc, int x, int y, char *txt, int off);
-int UpdateMenu(void); 
+int UpdateMenu(int sticks); 
+int mouse_moved = 0;
+int menu_aborted = 0;
 
 extern XContext MenuContext;
-
 /****************************************************************************
  *
  * Initiates a menu pop-up
  *
+ * Style = 1 = sticky menu, stays up on initial button release.
+ * Style = 0 = transient menu, drops on initial release.
  ***************************************************************************/
-int do_menu (MenuRoot *menu)
+int do_menu (MenuRoot *menu, int style)
 {
   int prevStashedX=0,prevStashedY=0;
   MenuRoot *PrevActiveMenu=0;
   MenuItem *PrevActiveItem=0;
   int retval=MENU_NOP;
   int x,y;
+  Time t0;
+  extern Time lastTimestamp;
+
 
   /* this condition could get ugly */
   if(menu->in_use)
@@ -108,6 +93,8 @@ int do_menu (MenuRoot *menu)
     }
   else
     {
+      mouse_moved = 0;
+      t0 = lastTimestamp;
       if(!GrabEm(MENU))
 	{
 	  XBell(dpy,Scr.screen);
@@ -118,7 +105,7 @@ int do_menu (MenuRoot *menu)
     }
   if (PopUpMenu (menu, x, y))
     {
-      retval = UpdateMenu();
+      retval = UpdateMenu(style);
     }
   else
     XBell (dpy, Scr.screen);
@@ -136,6 +123,10 @@ int do_menu (MenuRoot *menu)
       UngrabEm();
       WaitForButtonsUp();
     }
+  if(((lastTimestamp-t0) < 3*Scr.ClickTime)&&(mouse_moved==0))
+    menu_aborted = 1;
+  else
+    menu_aborted = 0;
   return retval;
 }
 
@@ -180,12 +171,12 @@ void RelieveHalfRectangle(Window win,int x,int y,int w,int h,
  ***********************************************************************/
 void PaintEntry(MenuRoot *mr, MenuItem *mi)
 {
-  int y_offset,text_y,d, y_height;
+  int y_offset,text_y,d, y_height,y,x;
   GC ShadowGC, ReliefGC, currentGC;
 
   y_offset = mi->y_offset;
   y_height = mi->y_height;
-  text_y = y_offset + Scr.StdFont.y;
+  text_y = y_offset + Scr.StdFont.y + mi->picture.height;
 
   ShadowGC = Scr.MenuShadowGC;
   if(Scr.d_depth<2)
@@ -200,7 +191,9 @@ void PaintEntry(MenuRoot *mr, MenuItem *mi)
 	XClearArea(dpy, mr->w, 0,y_offset-1,mr->width,y_height+2,0);
       else
 	XClearArea(dpy, mr->w, 0,y_offset+1,mr->width,y_height-1,0);
-      if ((mi->state)&&(mi->func != F_TITLE)&&(mi->func != F_NOP)&&*mi->item)
+      if ((mi->state)&&(mi->func_type != F_TITLE)&&
+	  (mi->func_type != F_NOP)&&(((*mi->item)!=0)||
+				     (mi->picture.picture != None)))
 	{
 	  RelieveRectangle(mr->w, 3, y_offset, mr->width-5, mi->y_height,
 			   ReliefGC,ShadowGC);
@@ -213,13 +206,16 @@ void PaintEntry(MenuRoot *mr, MenuItem *mi)
   else
     {
       XClearArea(dpy, mr->w, 0,y_offset,mr->width,y_height,0);
-      if ((mi->state)&&(mi->func != F_TITLE)&&(mi->func != F_NOP)&&*mi->item)
+      if ((mi->state)&&(mi->func_type != F_TITLE)&&
+	  (mi->func_type != F_NOP)&&(((*mi->item)!=0)||
+				     (mi->picture.picture != None)||	
+				     (mi->lpicture.picture != None)))
 	RelieveRectangle(mr->w, 2, y_offset, mr->width-4, mi->y_height,
 			 ReliefGC,ShadowGC);
       RelieveHalfRectangle(mr->w, 0, y_offset, mr->width,
 			   y_height, ReliefGC, ShadowGC);
     }
-  if(mi->func == F_TITLE)
+  if(mi->func_type == F_TITLE)
     {
       if(Scr.flags & MWMMenus)
 	{
@@ -246,7 +242,7 @@ void PaintEntry(MenuRoot *mr, MenuItem *mi)
     }
   else
     text_y += HEIGHT_EXTRA>>1;
-  if(mi->func == F_NOP && *mi->item==0) 
+  if(mi->func_type == F_NOP && *mi->item==0) 
     {
       if(Scr.flags & MWMMenus)
 	DrawSeparator(mr->w,ShadowGC,ReliefGC,2,y_offset-1+HEIGHT_SEPARATOR/2,
@@ -276,16 +272,74 @@ void PaintEntry(MenuRoot *mr, MenuItem *mi)
   if (mi->hotkey > 0)
     DrawUnderline(mr->w, currentGC,mi->x,text_y,mi->item,mi->hotkey - 1);
   if (mi->hotkey < 0)
-    DrawUnderline(mr->w, currentGC,mi->x2,text_y,mi->item2, -1 - mi->hotkey);
+   DrawUnderline(mr->w, currentGC,mi->x2,text_y,mi->item2, -1 - mi->hotkey);
 
   d=(Scr.EntryHeight-7)/2;
-  if(mi->func == F_POPUP)
+  if(mi->func_type == F_POPUP)
     if(mi->state)
       DrawTrianglePattern(mr->w, ShadowGC, ReliefGC, ShadowGC,mr->width-d-8,
 			  y_offset+d-1, mr->width-d-1, y_offset+d+7);
     else
       DrawTrianglePattern(mr->w, ReliefGC, ShadowGC, ReliefGC,mr->width-d-8,
 			  y_offset+d-1, mr->width-d-1, y_offset+d+7);
+
+  if(mi->picture.picture)
+    {
+      x = (mr->width - mi->picture.width)/2;
+      if((mi->lpicture.picture != None)&&(x < mr->width0+5))
+	{
+	  x = mr->width0+5;
+	}
+
+      if(mi->picture.depth > 1)
+	{
+	  Globalgcm = GCClipMask | GCClipXOrigin | GCClipYOrigin;
+	  Globalgcv.clip_mask = mi->picture.mask;
+	  Globalgcv.clip_x_origin= x;
+	  Globalgcv.clip_y_origin = y_offset+1;
+	  XChangeGC(dpy,ReliefGC,Globalgcm,&Globalgcv);
+	  XCopyArea(dpy,mi->picture.picture,mr->w,ReliefGC,	0,0,
+		    mi->picture.width, mi->picture.height,
+		    x,y_offset+1);
+	  Globalgcm = GCClipMask;
+	  Globalgcv.clip_mask = None;
+	  XChangeGC(dpy,ReliefGC,Globalgcm,&Globalgcv);
+	}
+      else
+	{
+	  XCopyPlane(dpy,mi->picture.picture,mr->w,
+		     currentGC,0,0,mi->picture.width,mi->picture.height,
+		     x,y_offset+1,1);
+	}
+    }
+
+  if(mi->lpicture.picture)
+    {
+      if((mi->picture.picture != None) && (*mi->item != 0))
+	y = y_offset + mi->y_height - mi->lpicture.height-1;
+      else 
+	y = y_offset + mi->y_height/2 - mi->lpicture.height/2;
+      if(mi->lpicture.depth > 1)
+	{
+	  Globalgcm = GCClipMask | GCClipXOrigin | GCClipYOrigin;
+	  Globalgcv.clip_mask = mi->lpicture.mask;
+	  Globalgcv.clip_x_origin= 5;
+	  Globalgcv.clip_y_origin = y;
+
+	  XChangeGC(dpy,ReliefGC,Globalgcm,&Globalgcv);
+	  XCopyArea(dpy,mi->lpicture.picture,mr->w,ReliefGC,0,0,
+		    mi->lpicture.width, mi->lpicture.height,5,y);
+	  Globalgcm = GCClipMask;
+	  Globalgcv.clip_mask = None;
+	  XChangeGC(dpy,ReliefGC,Globalgcm,&Globalgcv);
+	}
+      else
+	{
+	  XCopyPlane(dpy,mi->lpicture.picture,mr->w,
+		     currentGC,0,0,mi->lpicture.width,mi->lpicture.height,
+		     5,y,1);
+	}
+    }
   return;
 }
 
@@ -303,7 +357,6 @@ void  DrawUnderline(Window w, GC gc, int x, int y, char *txt, int posn)
 {
   int off1 = XTextWidth(Scr.StdFont.font, txt, posn);
   int off2 = XTextWidth(Scr.StdFont.font, txt, posn + 1) - 1;
-  
   XDrawLine(dpy, w, gc, x + off1, y + 2, x + off2, y + 2);
 }
 /****************************************************************************
@@ -411,8 +464,9 @@ int FindEntry(void)
 
   if(ActiveItem)
     {
+      MenuRoot *menu;
       /* create a new sub-menu */
-      if((ActiveItem->func == F_POPUP)&&
+      if((ActiveItem->func_type == F_POPUP)&&
 	 ((Scr.flags & MWMMenus)||(x>(3*ActiveMenu->width>>2))))
 	{
 	  PrevPrevMenu = PrevMenu;
@@ -421,18 +475,24 @@ int FindEntry(void)
 	  PrevY = MenuY;
 	  PrevMenu = ActiveMenu;
 	  PrevItem = ActiveItem;
-	  retval = do_menu(ActiveItem->menu);
-	  /* Unfortunately, this is needed (why?) for multi-screen operation */
-	  flush_expose(ActiveMenu->w);
-	  for (mi = ActiveMenu->first; mi != NULL; mi = mi->next)
+	  /*******************************************/
+	  menu = FindPopup(&ActiveItem->action[5]);
+	  if(menu != NULL)
 	    {
-	      PaintEntry(ActiveMenu, mi);
+	      retval = do_menu(menu,0);
+	      /* Unfortunately, this is needed (why?) for multi-screen 
+		 operation */
+	      /* flush_expose(ActiveMenu->w);*/
+	      for (mi = ActiveMenu->first; mi != NULL; mi = mi->next)
+		{
+		  PaintEntry(ActiveMenu, mi);
+		}
+	      XSync(dpy, 0);
+	      MenuY = PrevY;
+	      PrevMenu = PrevPrevMenu;
+	      PrevItem = PrevPrevItem;
+	      PrevY = PrevPrevY;
 	    }
-	  XSync(dpy, 0);
-	  MenuY = PrevY;
-	  PrevMenu = PrevPrevMenu;
-	  PrevItem = PrevPrevItem;
-	  PrevY = PrevPrevY;
 	}
     }
   /* end a sub-menu */ 
@@ -516,19 +576,29 @@ void menuShortcuts(XEvent *ev)
  *  Procedure:
  *	Updates menu display to reflect the highlighted item
  * 
+ *  Input
+ *      sticks = 0, transient style menu, drops on button release
+ *      sticks = 1, sticky style, stays up if initial release is close to initial press.
  *  Returns:
  *      0 on error condition
  *      1 on return from submenu to parent menu
  *      2 on button release return
  *
  ***********************************************************************/
-int UpdateMenu(void)
+int UpdateMenu(int sticks)
 {
-  int done,func;
+  int done;
   int retval;
+  MenuItem *InitialMI;
   MenuRoot *actual_mr;
+  int x_init, y_init,x,y;
+
+  XQueryPointer( dpy, Scr.Root, &JunkRoot, &JunkChild,
+		&JunkX,&JunkY, &x_init, &y_init, &JunkMask);
 
   FindEntry();
+  InitialMI = ActiveItem;
+
   while (TRUE)
     {
       /* block until there is an event */
@@ -551,6 +621,11 @@ int UpdateMenu(void)
 	{
 	case ButtonRelease:
 	  /* The following lines holds the menu when the button is released */
+	  if(sticks)
+	    {
+	      sticks = 0;
+	      break;
+	    }
 	  if ((Scr.flags & MWMMenus) && !ActiveItem && (menu_on > 1))
 	    {
 	      int x,y;
@@ -567,27 +642,9 @@ int UpdateMenu(void)
 	  PopDownMenu();
 	  if(ActiveItem)
 	    {
-	      func = ActiveItem->func;
 	      done = 1;
-	      if(ButtonWindow)
-		{
-		  ExecuteFunction(func, ActiveItem->action,
-				  ButtonWindow->frame,
-				  ButtonWindow, &Event, Context,
-				  ActiveItem->val1,ActiveItem->val2,
-				  ActiveItem->val1_unit,ActiveItem->val2_unit,
-				  ActiveItem->menu,-1);
-		}
-	      else
-		{
-		  ExecuteFunction(func,ActiveItem->action,
-				  None,None, &Event, 
-				  Context,ActiveItem->val1,
-				  ActiveItem->val2,
-				  ActiveItem->val1_unit,ActiveItem->val2_unit,
-				  ActiveItem->menu,-1);
-
-		}
+	      ExecuteFunction(ActiveItem->action,
+			      ButtonWindow, &Event, Context,-1);
 	    }
 	  ActiveItem = NULL;
 	  ActiveMenu = NULL;
@@ -601,9 +658,16 @@ int UpdateMenu(void)
 	  break;
 
 	case MotionNotify:
+	  XQueryPointer( dpy, Scr.Root, &JunkRoot, &JunkChild,
+			&JunkX,&JunkY, &x, &y, &JunkMask);
+	  if(((x-x_init > 3)||(x_init-x > 3))&&
+	     ((y-y_init > 3)||(y_init-y > 3)))
+	    mouse_moved = 1;
 	  done = 1;
 
 	  retval = FindEntry();
+	  if(ActiveItem != InitialMI)
+	    sticks = 0;
 	  if((retval == MENU_DONE)||(retval == SUBMENU_DONE))
 	    {
 	      PopDownMenu();	      
@@ -673,13 +737,13 @@ Bool PopUpMenu (MenuRoot *menu, int x, int y)
     {
       y = Tmp_win->frame_y+Tmp_win->boundary_width+Tmp_win->title_height+1;
       x = Tmp_win->frame_x + Tmp_win->boundary_width + 
-	Button*Tmp_win->title_height+1;
+	ButtonPosition(Context,Tmp_win)*Tmp_win->title_height+1;
     }
   if((Tmp_win)&&(menu_on==1)&&(Context&C_RALL))
     {
       y = Tmp_win->frame_y+Tmp_win->boundary_width+Tmp_win->title_height+1;
       x = Tmp_win->frame_x +Tmp_win->frame_width - Tmp_win->boundary_width-
-	Button*Tmp_win->title_height - menu->width+1;
+	ButtonPosition(Context,Tmp_win)*Tmp_win->title_height - menu->width+1;
     }
   if((Tmp_win)&&(menu_on==1)&&(Context&C_TITLE))
     {
@@ -687,7 +751,7 @@ Bool PopUpMenu (MenuRoot *menu, int x, int y)
       if(x < Tmp_win->frame_x + Tmp_win->title_x)
 	x = Tmp_win->frame_x + Tmp_win->title_x;
       if((x + menu->width) >
-	 (Tmp_win->frame_x + Tmp_win->title_x +Tmp_win->title_width))
+	 (Tmp_win->frame_x + Tmp_win->title_x +Tmp_win->title_width))	
 	x = Tmp_win->frame_x + Tmp_win->title_x +Tmp_win->title_width-
 	  menu->width +1;
     }
@@ -781,3 +845,472 @@ void WaitForButtonsUp()
 
 
 
+
+void DestroyMenu(MenuRoot *mr)
+{
+  MenuItem *mi,*tmp2;
+  MenuRoot *tmp, *prev;
+
+  if(mr == NULL)
+    return;
+
+  tmp = Scr.AllMenus;
+  prev = NULL;
+  while((tmp != NULL)&&(tmp != mr))
+    {
+      prev = tmp;
+      tmp = tmp->next;
+    }
+  if(tmp != mr)
+    return;
+
+  if(prev == NULL)
+    Scr.AllMenus = mr->next;
+  else
+    prev->next = mr->next;
+
+  XDestroyWindow(dpy,mr->w);
+  XDeleteContext(dpy, mr->w, MenuContext);  
+  
+  /* need to free the window list ? */
+  mi = mr->first;
+  while(mi != NULL)
+    {
+      tmp2 = mi->next;
+      if (mi->item != NULL) free(mi->item);
+      if (mi->item2 != NULL) free(mi->item2);
+      if (mi->action != NULL) free(mi->action);
+      DestroyPicture(dpy,&mi->picture);
+      DestroyPicture(dpy,&mi->lpicture);
+      free(mi);
+      mi = tmp2;
+    }
+  free(mr);
+}
+
+/****************************************************************************
+ * 
+ * Generates the windows for all menus
+ *
+ ****************************************************************************/ 
+void MakeMenus(void)
+{
+  MenuRoot *mr;
+
+  mr = Scr.AllMenus;
+  while(mr != NULL)
+    {
+      MakeMenu(mr);
+      mr = mr->next;
+    }
+}
+
+/****************************************************************************
+ * 
+ * Generates the window for a menu
+ *
+ ****************************************************************************/ 
+void MakeMenu(MenuRoot *mr)
+{
+  MenuItem *cur;
+  unsigned long valuemask;
+  XSetWindowAttributes attributes;
+  int y,width;
+
+  if((mr->func != F_POPUP)||(!Scr.flags & WindowsCaptured))
+    return;
+
+  /* allow two pixels for top border */
+  mr->width = 0;
+  mr->width2 = 0;
+  mr->width0 = 0;
+  for (cur = mr->first; cur != NULL; cur = cur->next)
+    {
+      width = XTextWidth(Scr.StdFont.font, cur->item, cur->strlen);
+      if(width < cur->picture.width)
+	width = cur->picture.width;
+      if(cur->func_type == F_POPUP)
+	width += 15;
+      if (width <= 0)
+	width = 1;
+      if (width > mr->width)
+	mr->width = width;
+
+      width = XTextWidth(Scr.StdFont.font, cur->item2, cur->strlen2);
+      if (width < 0)
+	width = 0;
+      if (width > mr->width2)
+	mr->width2 = width;
+      if((width==0)&&(cur->strlen2>0))
+	mr->width2 = 1;
+
+      if(cur->lpicture.picture != None)
+	if(mr->width0 < (cur->lpicture.width+3))
+	  {
+	    mr->width0 = cur->lpicture.width+3;
+	  }
+	    
+    }
+
+  /* lets first size the window accordingly */
+  mr->width += 10;
+  if(mr->width2 > 0)
+    mr->width += 5;
+  
+
+  for (y=2, cur = mr->first; cur != NULL; cur = cur->next)
+    {
+      cur->y_offset = y;
+      cur->x = 5+mr->width0;
+      if(cur->func_type==F_TITLE)
+	{
+	  /* Title */
+	  if(cur->strlen2  == 0)
+	    cur->x = (mr->width+mr->width2+mr->width0
+		      - XTextWidth(Scr.StdFont.font, cur->item,
+					     cur->strlen))>>1;
+	  
+	  if((cur->strlen > 0)||(cur->strlen2>0))
+	    {
+	      if(Scr.flags & MWMMenus)
+		cur->y_height = Scr.EntryHeight + HEIGHT_EXTRA_TITLE;
+	      else
+		{
+		  if((cur == mr->first)||(cur->next == NULL))
+		    cur->y_height=Scr.EntryHeight-HEIGHT_EXTRA+1+
+		      (HEIGHT_EXTRA_TITLE>>1);
+		  else
+		    cur->y_height = Scr.EntryHeight -HEIGHT_EXTRA +1+
+		      HEIGHT_EXTRA_TITLE;
+		}
+	    }
+	  else
+	    cur->y_height = HEIGHT_SEPARATOR;
+	}
+      else if((cur->strlen==0)&&(cur->strlen2 == 0))
+	/* Separator */
+	cur->y_height = HEIGHT_SEPARATOR;
+      else
+	/* Normal text entry */
+	cur->y_height = Scr.EntryHeight;
+      cur->y_height += cur->picture.height;
+      if(cur->y_height < cur->lpicture.height+4)
+	cur->y_height = cur->lpicture.height+4;
+      y += cur->y_height;
+      if(mr->width2 == 0)
+	{
+	  cur->x2 = cur->x;
+	}
+      else
+	{
+	  cur->x2 = mr->width -5 + mr->width0;
+	}
+    }
+  mr->in_use = 0;
+  mr->height = y+2;
+  
+  
+#ifndef NO_SAVEUNDERS   
+  valuemask = (CWBackPixel | CWEventMask | CWCursor | CWSaveUnder);
+#else
+  valuemask = (CWBackPixel | CWEventMask | CWCursor);
+#endif
+  attributes.background_pixel = Scr.MenuColors.back;
+  attributes.event_mask = (ExposureMask | EnterWindowMask);
+  attributes.cursor = Scr.FvwmCursors[MENU];
+#ifndef NO_SAVEUNDERS   
+  attributes.save_under = TRUE;
+#endif
+  if(mr->w != None)
+    XDestroyWindow(dpy,mr->w);
+
+  mr->width = mr->width0 + mr->width + mr->width2;
+  mr->w = XCreateWindow (dpy, Scr.Root, 0, 0, (unsigned int) (mr->width),
+			 (unsigned int) mr->height, (unsigned int) 0,
+			 CopyFromParent, (unsigned int) InputOutput,
+			 (Visual *) CopyFromParent,
+			 valuemask, &attributes);
+  XSaveContext(dpy,mr->w,MenuContext,(caddr_t)mr);
+  
+  return;
+}
+
+/***********************************************************************
+ * Procedure:
+ *	scanForHotkeys - Look for hotkey markers in a MenuItem
+ * 							(pete@tecc.co.uk)
+ * 
+ * Inputs:
+ *	it	- MenuItem to scan
+ * 	which 	- +1 to look in it->item1 and -1 to look in it->item2.
+ *
+ ***********************************************************************/
+void scanForHotkeys(MenuItem *it, int which) 
+{
+  char *start, *txt;
+
+  start = (which > 0) ? it->item : it->item2;	/* Get start of string	*/
+  for (txt = start; *txt != '\0'; txt++) 
+    {
+      /* Scan whole string	*/
+      if (*txt == '&') 
+	{		/* A hotkey marker?			*/
+	  if (txt[1] == '&') 
+	    {	/* Just an escaped &			*/
+	      char *tmp;		/* Copy the string down over it	*/
+	      for (tmp = txt; *tmp != '\0'; tmp++) tmp[0] = tmp[1];
+	      continue;		/* ...And skip to the key char		*/
+	    }
+	  /* It's a hot key marker - work out the offset value		*/
+	  it->hotkey = (1 + (txt - start)) * which;
+	  for (; *txt != '\0'; txt++) txt[0] = txt[1];	/* Copy down..	*/
+	  return;			/* Only one hotkey per item...	*/
+	}
+    }
+  it->hotkey = 0;		/* No hotkey found.  Set offset to zero	*/
+}
+
+
+
+void scanForPixmap(char *instring, Picture *p, char identifier) 
+{
+  char *tstart, *txt, *save_instring;
+  int i;
+  char name[100];
+  extern char *IconPath;
+#ifdef XPM
+  extern char *PixmapPath;
+#endif
+
+  /* save instring in case can't find pixmap */
+  save_instring = (char *)safemalloc(strlen(instring)+1);
+  strcpy(save_instring,instring);
+
+  InitPicture(p);
+  /* Scan whole string	*/
+  for (txt = instring; *txt != '\0'; txt++) 
+    {
+      /* A hotkey marker? */
+      if (*txt == identifier) 
+	{
+	  /* Just an escaped &	*/
+	  if (txt[1] == identifier) 
+	    {	
+	      char *tmp;		/* Copy the string down over it	*/
+	      for (tmp = txt; *tmp != '\0'; tmp++) tmp[0] = tmp[1];
+	      continue;		/* ...And skip to the key char		*/
+	    }
+	  /* It's a hot key marker - work out the offset value		*/
+	  tstart = txt;
+	  txt++;
+	  i=0;
+	  while((*txt != identifier)&&(*txt != '\0')&&(i<99))
+	    {
+	      name[i] = *txt;
+	      txt++;
+	      i++;
+	    }
+	  name[i] = 0;
+	  /* Next, check for a color pixmap */
+#ifdef XPM
+	  GetPicture(dpy,Scr.Root,IconPath,PixmapPath,name,p);
+#else
+	  GetPicture(dpy,Scr.Root,IconPath,IconPath,name,p);
+#endif
+	  if(*txt != '\0')txt++;
+	  while(*txt != '\0')
+	    {
+	      *tstart++ = *txt++;
+	    }
+	  *tstart = 0;
+          if (p->picture == None)
+            strcpy(instring,save_instring);
+          free(save_instring);
+	  return;		/* Only one hotkey per item...		*/
+	}
+    }
+}
+
+
+
+/***********************************************************************
+ *
+ *  Procedure:
+ *	AddToMenu - add an item to a root menu
+ *
+ *  Returned Value:
+ *	(MenuItem *)
+ *
+ *  Inputs:
+ *	menu	- pointer to the root menu to add the item
+ *	item	- the text to appear in the menu
+ *	action	- the string to possibly execute
+ *	func	- the numeric function
+ *
+ * ckh - need to add boolean to say whether or not to expand for pixmaps,
+ *       so built in window list can handle windows w/ * and % in title.
+ *
+ ***********************************************************************/
+void AddToMenu(MenuRoot *menu, char *item, char *action)
+{
+  MenuItem *tmp;
+  char *start,*end;
+
+  if(item == NULL)
+     return;
+
+  tmp = (MenuItem *)safemalloc(sizeof(MenuItem));
+  if (menu->first == NULL)
+    {
+      menu->first = tmp;
+      tmp->prev = NULL;
+    }
+  else
+    {
+      menu->last->next = tmp;
+      tmp->prev = menu->last;
+    }
+  menu->last = tmp;
+
+  /* skip leading spaces */
+  /*while(isspace(*item)&&(item != NULL))
+    item++;*/
+  /* up to first tab goes in "item" field */
+  start = item;
+  end=item;
+  while((*end != '\t')&&(*end != 0))
+    end++;
+  tmp->item = safemalloc(end-start+1);
+  strncpy(tmp->item,start,end-start);
+  tmp->item[end-start] = 0;
+  tmp->item2 = NULL;
+  if(*end=='\t')
+    {
+      start = end+1;
+      while(*end != 0)
+	end++;  
+      if(end-start != 0)
+	{
+	  tmp->item2 = safemalloc(end-start+1);
+	  strncpy(tmp->item2,start,end-start);
+	  tmp->item2[end-start] = 0;
+	}
+    }
+
+  if (item != (char *)0)
+    {
+      scanForPixmap(tmp->item,&tmp->picture,'*');
+      scanForPixmap(tmp->item,&tmp->lpicture,'%');
+      scanForHotkeys(tmp, 1);				/* pete@tecc.co.uk */
+      tmp->strlen = strlen(tmp->item);
+    }
+  else
+    tmp->strlen = 0;
+
+  if (tmp->item2 != (char *)0)
+    {
+      if(tmp->picture.picture == None)
+        scanForPixmap(tmp->item2,&tmp->picture,'*');
+      if(tmp->lpicture.picture == None)
+        scanForPixmap(tmp->item2,&tmp->lpicture,'%');
+      if (tmp->hotkey == 0) 
+	scanForHotkeys(tmp, -1);	/* pete@tecc.co.uk */
+      tmp->strlen2 = strlen(tmp->item2);
+    }
+  else
+    tmp->strlen2 = 0;
+
+  tmp->action = stripcpy(action);
+  tmp->next = NULL;
+  tmp->state = 0;
+  tmp->func_type = find_func_type(tmp->action);
+  tmp->item_num = menu->items++;
+}
+
+/***********************************************************************
+ *
+ *  Procedure:
+ *	NewMenuRoot - create a new menu root
+ *
+ *  Returned Value:
+ *	(MenuRoot *)
+ *
+ *  Inputs:
+ *	name	- the name of the menu root
+ *
+ ***********************************************************************/
+MenuRoot *NewMenuRoot(char *name,int junk)
+{
+  MenuRoot *tmp;
+  
+  tmp = (MenuRoot *) safemalloc(sizeof(MenuRoot));
+  if(junk == 0)
+    tmp->func = F_POPUP;
+  else
+    tmp->func = F_FUNCTION;
+
+  tmp->name = stripcpy(name);
+  tmp->first = NULL;
+  tmp->last = NULL;
+  tmp->items = 0;
+  tmp->width = 0;
+  tmp->width2 = 0;
+  tmp->w = None;
+  tmp->next  = Scr.AllMenus;
+  Scr.AllMenus = tmp;
+  return (tmp);
+}
+
+
+
+/***********************************************************************
+ * change by KitS@bartley.demon.co.uk to correct popups off title buttons
+ *
+ *  Procedure:
+ *ButtonPosition - find the actual position of the button
+ *                 since some buttons may be disabled
+ *
+ *  Returned Value:
+ *The button count from left or right taking in to account
+ *that some buttons may not be enabled for this window
+ *
+ *  Inputs:
+ *      context - context as per the global Context
+ *      t       - the window (FvwmWindow) to test against
+ *
+ ***********************************************************************/
+int ButtonPosition(int context, FvwmWindow * t)
+{
+  int i;
+  int buttons = -1;
+  
+  if (context&C_RALL)
+    {
+      for(i=0;i<Scr.nr_right_buttons;i++)
+	{
+	  if(t->right_w[i])
+	    {
+	      buttons++;
+	    }
+	  /* is this the button ? */
+	  if (((1<<i)*C_R1) & context)
+	    return(buttons);
+	}
+    }
+  else
+    {
+      for(i=0;i<Scr.nr_left_buttons;i++)
+	{
+	  if(t->left_w[i])
+	    {
+	      buttons++;
+	    }
+	  /* is this the button ? */
+	  if (((1<<i)*C_L1) & context)
+	    return(buttons);
+	}
+    }
+  /* you never know... */
+  return 0;
+}
+
