@@ -1,6 +1,6 @@
 /****************************************************************************
  * This module is all original code 
- * by Rob Nation
+ * by Rob Nation 
  * Copyright 1993, Robert Nation
  *     You may use this code for any purpose, as long as the original
  *     copyright remains in the source code and all documentation
@@ -36,6 +36,8 @@ int npipes;
 int *readPipes;
 int *writePipes;
 int *pipeOn;
+char **pipeName;
+
 unsigned long *PipeMask;
 struct queue_buff_struct **pipeQueue;
 
@@ -53,6 +55,7 @@ void initModules(void)
   readPipes = (int *)safemalloc(sizeof(int)*npipes);
   pipeOn = (int *)safemalloc(sizeof(int)*npipes);
   PipeMask = (unsigned long *)safemalloc(sizeof(unsigned long)*npipes);
+  pipeName = (char **)safemalloc(sizeof(char *)*npipes);
   pipeQueue=(struct queue_buff_struct **)
     safemalloc(sizeof(struct queue_buff_struct *)*npipes);
 
@@ -63,7 +66,7 @@ void initModules(void)
       pipeOn[i] = -1;
       PipeMask[i] = MAX_MASK;
       pipeQueue[i] = (struct queue_buff_struct *)NULL;
-
+      pipeName[i] = NULL;
     }
 }
 
@@ -77,6 +80,11 @@ void ClosePipes(void)
 	  close(writePipes[i]);
 	  close(readPipes[i]);
 	}
+      if(pipeName[i] != NULL)
+	{
+	  free(pipeName[i]);
+	  pipeName[i] = 0;
+	}
       while(pipeQueue[i] != NULL)
 	{
 	  DeleteQueueBuff(i);
@@ -84,54 +92,48 @@ void ClosePipes(void)
     }
 }
 
-void executeModule(char *action,FILE *fd, char **win, int *context)
+
+void executeModule(XEvent *eventp,Window w,FvwmWindow *tmp_win,
+		     unsigned long context, char *action,int* Module)
 {
   int fvwm_to_app[2],app_to_fvwm[2];
-  int i,val;
-  char command[256];
+  int i,val,nargs = 0;
   char *cptr;
-  char *aptr;
-  char *args[10];
+  char *args[20];
   char *arg1 = NULL;
-  char arg2[40];
-  char arg3[40];
-  char arg5[40];
-  char arg6[40];
-  char *end;
+  char arg2[20];
+  char arg3[20];
+  char arg5[20];
+  char arg6[20];
   extern char *ModulePath;
-  extern FILE *config_fd;
   extern char *fvwm_file;
+  Window win;
+
+  if(eventp->type != KeyPress)
+    UngrabEm();
 
   if(action == NULL)
     return;
-  strcpy(command,action);
 
-  cptr = command;
-  while((isspace(*cptr))&&(*cptr != '\n')&&(*cptr != 0))
-    cptr++;
-
-  end = cptr;
-  while((!(isspace(*end))&&(*end != '\n'))&&(*end != 0)&&(end <(command+256)))
-    end++;
-
-  if((*end == 0)||(end >= command+256))
-    aptr = NULL;
+  if(tmp_win)
+    win = tmp_win->w;
   else
-    aptr = end+1;
-  *end = 0;
+    win = None;
 
-  if(aptr)
-    {
-      while((isspace(*aptr)||(*aptr=='\n'))&&(*aptr!=0)&&(aptr<(command+256)))
-	aptr++;
-      if((*aptr == 0)||(*aptr == '\n'))
-	aptr = NULL;
-    }
+  /* If we execute a module, don't wait for buttons to come up,
+   * that way, a pop-up menu could be implemented */
+  *Module = 0;
+
+  action = GetNextToken(action, &cptr);
 
   arg1 = findIconFile(cptr,ModulePath,X_OK);
   if(arg1 == NULL)
     {
-      fprintf(stderr,"Fvwm: No such module %s %s\n",ModulePath,cptr);
+      if(cptr != NULL)
+	{
+	  fvwm_err("No such module %s %s",ModulePath,cptr,NULL);
+	  free(cptr);
+	}
       return;
     }
 
@@ -141,7 +143,9 @@ void executeModule(char *action,FILE *fd, char **win, int *context)
     i++;
   if(i>=npipes)
     {
-      fprintf(stderr,"fvwm: Too many Accessories!\n");
+      fvwm_err("Too many Accessories!",NULL,NULL,NULL);
+      if(cptr != NULL)
+	free(cptr);
       return;
     }
   
@@ -150,18 +154,51 @@ void executeModule(char *action,FILE *fd, char **win, int *context)
    * I can detect when the module crashes/malfunctions */
   if(pipe(fvwm_to_app)!=0)
     {
-      fprintf(stderr,"Fvwm: Failed to open pipe\n");
+      fvwm_err("Failed to open pipe",NULL,NULL,NULL);
       return;
     }
   if(pipe(app_to_fvwm)!=0)
     {
-      fprintf(stderr,"Fvwm: Failed to open pipe2\n");
+      fvwm_err("Failed to open pipe2",NULL,NULL,NULL);
+      if(cptr != NULL)
+	free(cptr);
       close(fvwm_to_app[0]);
       close(fvwm_to_app[1]);
       return;
     }
 
-  
+  pipeName[i] = stripcpy(cptr);
+  sprintf(arg2,"%d",app_to_fvwm[1]);
+  sprintf(arg3,"%d",fvwm_to_app[0]);
+  sprintf(arg5,"%lx",(unsigned long)win);
+  sprintf(arg6,"%lx",(unsigned long)context);
+  args[0]=arg1;
+  args[1]=arg2;
+  args[2]=arg3;
+  if(fvwm_file != NULL)
+    args[3]=fvwm_file;
+  else
+    args[3]="none";
+  args[4]=arg5;
+  args[5]=arg6;
+  nargs = 6;
+  while((action != NULL)&&(nargs < 20)&&(strlen(args[nargs-1]) > 0))
+    {
+      args[nargs] = 0;
+      action = GetNextToken(action,&args[nargs]);
+      nargs++;
+    }
+  if(strlen(args[nargs-1]) <= 0)
+    {
+      nargs--;
+      if(args[nargs] != NULL)
+	free(args[nargs]);
+    }
+  args[nargs] = 0;
+
+  /* Try vfork instead of fork. The man page says that vfork is better! */
+  /* Also, had to change exit to _exit() */
+  /* Not everyone has vfork! */
   val = fork();
   if(val > 0)
     {
@@ -188,38 +225,24 @@ void executeModule(char *action,FILE *fd, char **win, int *context)
 	fvwm_err("module close-on-exec failed",NULL,NULL,NULL);
       if (fcntl(writePipes[i], F_SETFD, 1) == -1) 
 	fvwm_err("module close-on-exec failed",NULL,NULL,NULL);
+      for(i=6;i<nargs;i++)
+	{
+	  if(args[i] != 0)
+	    free(args[i]);
+	}
+	
+      if(cptr != NULL)
+	free(cptr);
     }
   else if (val ==0)
     {
       /* this is  the child */
-      /* need to close config_fd if its still open! */
-      if(config_fd != (FILE *)NULL)
-	/* Fixes some funny stuff with svr4 and stream IO */
-	/* fclose(config_fd) */
-	close(fileno(config_fd));
-
       /* this fork execs the module */
       close(fvwm_to_app[1]);
       close(app_to_fvwm[0]);
-      sprintf(arg2,"%d",app_to_fvwm[1]);
-      sprintf(arg3,"%d",fvwm_to_app[0]);
-      sprintf(arg5,"%lx",(unsigned long)win);
-      sprintf(arg6,"%lx",(unsigned long)context);
-      args[0]=arg1;
-      args[1]=arg2;
-      args[2]=arg3;
-      args[3]=fvwm_file;
-      args[4]=arg5;
-      args[5]=arg6;
-      if(aptr != NULL)
-	{
-	  args[6] = aptr;
-	  args[7] = 0;
-	}
-      else
-	args[6]= 0;
+      
       execvp(arg1,args);
-      fprintf(stderr,"Execution of module failed: %s",arg1);      
+      fvwm_err("Execution of module failed: %s",arg1,NULL,NULL);   
       perror("");
       close(app_to_fvwm[1]);
       close(fvwm_to_app[0]);
@@ -227,7 +250,7 @@ void executeModule(char *action,FILE *fd, char **win, int *context)
     }
   else
     {
-      fprintf(stderr,"Fork failed\n");
+      fvwm_err("Fork failed",NULL,NULL,NULL);
       free(arg1);
     }
   return;
@@ -238,12 +261,11 @@ void HandleModuleInput(Window w, int channel)
   char text[256];
   int size;
   int cont,n;
-  char *newaction = NULL;
 
   /* Already read a (possibly NULL) window id from the pipe,
    * Now read an fvwm bultin command line */
-  n = read(readPipes[channel], &size, sizeof(int));
-  if(n < sizeof(int))
+  n = read(readPipes[channel], &size, sizeof(size));
+  if(n < sizeof(size))
     {
       KillModule(channel,1);
       return;
@@ -251,7 +273,7 @@ void HandleModuleInput(Window w, int channel)
 
   if(size >255)
     {
-      fprintf(stderr,"Module command is too big (%d)\n",size);
+      fvwm_err("Module command is too big (%d)",(void *)size,NULL,NULL);
       size=255;
     }
 
@@ -265,8 +287,8 @@ void HandleModuleInput(Window w, int channel)
     }
   
   text[n]=0;
-  n = read(readPipes[channel],&cont, sizeof(int));
-  if(n < sizeof(int))
+  n = read(readPipes[channel],&cont, sizeof(cont));
+  if(n < sizeof(cont))
     {
       KillModule(channel,3);
       return;
@@ -277,70 +299,11 @@ void HandleModuleInput(Window w, int channel)
     }
   if(strlen(text)>0)
     {
-      char function[256],*ptr;
-      MenuRoot *mr=0;
-      char *item=NULL;
-      extern int func_val_1,func_val_2,func,Context;
-      extern struct config func_config[];
-      extern unsigned PopupCount;
-      extern MenuRoot *PopupTable[MAXPOPUPS];      
+      extern int Context;
       FvwmWindow *tmp_win;
-      extern char *orig_tline;
-      int n,unit_val_1,unit_val_2;
-      char unit_1, unit_2;
 
-
-      orig_tline = text;
       Event.xany.type = ButtonRelease;
       Event.xany.window = w;
-	
-      func_val_1 = 0;
-      func_val_2 = 0;
-      unit_1 = 's';
-      unit_2 = 's';
-      n = sscanf(text,"%s %d %d",function,&func_val_1,&func_val_2);
-      if(n != 3)
-	n = sscanf(text,"%s %d%c %d%c",function,&func_val_1,&unit_1,&func_val_2,&unit_2);
-  
-      if(mystrcasecmp(function,"SET_MASK")==0)
-	{
-	  PipeMask[channel] = func_val_1;
-	  return;
-	}
-      func = F_NOP;
-      match_string(func_config,function,"bad mouse function:",NULL);
-      if((func == F_POPUP)||(func == F_FUNCTION))
-	{
-	  unsigned i;
-	  ptr = stripcpy2(text,0,True);
-	  if(ptr != NULL)
-	    for (i = 0; i < PopupCount; i++)
-	      if (mystrcasecmp(PopupTable[i]->name,ptr) == 0)
-		{
-		  mr = PopupTable[i];
-		  break;
-		}
-	  if (!mr)
-	    {
-	      no_popup(ptr);
-	      func = F_NOP;
-	    }
-	}
-      else if((func == F_EXEC)||(func == F_RESTART)||
-	      (func == F_CIRCULATE_UP)||(func == F_CIRCULATE_DOWN)||
-	      (func == F_WARP)||(func == F_MODULE))
-	{
-	  if((func == F_EXEC)||(func == F_RESTART)||(func== F_MODULE))
-	    {
-	      item = stripcpy2(text,0,True);
-	      newaction = stripcpy3(text,True);
-	    }
-	  else
-	    {
-	      item = stripcpy2(text,0,False);
-	      newaction = stripcpy3(text,False);
-	    }
-	}
       if (XFindContext (dpy, w, FvwmContext, (caddr_t *) &tmp_win) == XCNOENT)
 	{
 	  tmp_win = NULL;
@@ -364,18 +327,8 @@ void HandleModuleInput(Window w, int channel)
 	  Event.xbutton.y = 0;
 	  Event.xbutton.subwindow = None;
 	}
-      if(unit_1 == 'p')
-	unit_val_1 = 100;
-      else
-	unit_val_1 = Scr.MyDisplayWidth;
-      if(unit_2 == 'p')
-	unit_val_2 = 100;
-      else
-	unit_val_2 = Scr.MyDisplayHeight;
-
       Context = GetContext(tmp_win,&Event,&w);
-      ExecuteFunction(func,newaction, w, tmp_win, &Event, Context,
-		      func_val_1,func_val_2,unit_val_1,unit_val_2,mr,channel);
+      ExecuteFunction(text,tmp_win,&Event,Context ,channel); 
     }
   return;
 }
@@ -399,6 +352,30 @@ void KillModule(int channel, int place)
     {
       DeleteQueueBuff(channel);
     }
+  if(pipeName[channel] != NULL)
+    {
+      free(pipeName[channel]);
+      pipeName[channel] = NULL;
+    }
+	
+  return;
+}
+
+void KillModuleByName(char *name)
+{
+  int i = 0;
+
+  if(name == NULL)
+    return;
+  
+  while(i<npipes)
+    {
+      if((pipeName[i] != NULL)&&(matchWildcards(name,pipeName[i])))
+	{
+	  KillModule(i,10);	
+	}
+      i++;
+    }
   return;
 }
 
@@ -409,29 +386,30 @@ void Broadcast(unsigned long event_type, unsigned long num_datum,
 	       unsigned long data7)
 {
   int i;
-  unsigned long body[10];
+  unsigned long body[11];
+  extern Time lastTimestamp;
 
   body[0] = START_FLAG;
   body[1] = event_type;
-  body[2] = num_datum+3;
-    
+  body[2] = num_datum+HEADER_SIZE;
+  body[3] = lastTimestamp;
   if(num_datum>0)
-    body[3] = data1;
+    body[HEADER_SIZE] = data1;
   if(num_datum>1)
-    body[4] = data2;
+    body[HEADER_SIZE+1] = data2;
   if(num_datum>2)
-    body[5] = data3;
+    body[HEADER_SIZE+2] = data3;
   if(num_datum>3)
-    body[6] = data4;
+    body[HEADER_SIZE+3] = data4;
   if(num_datum>4)
-    body[7] = data5;
+    body[HEADER_SIZE+4] = data5;
   if(num_datum>5)
-    body[8] = data6;
+    body[HEADER_SIZE+5] = data6;
   if(num_datum>6)
-    body[9] = data7;
+    body[HEADER_SIZE+6] = data7;
 
   for(i=0;i<npipes;i++)   
-    PositiveWrite(i,body, (num_datum+3)*sizeof(unsigned long));
+    PositiveWrite(i,body, (num_datum+4)*sizeof(body[0]));
 }
 
 
@@ -442,62 +420,65 @@ void SendPacket(int module, unsigned long event_type, unsigned long num_datum,
 	       unsigned long data4, unsigned long data5, unsigned long data6,
 	       unsigned long data7)
 {
-  unsigned long body[10];
+  unsigned long body[11];
+  extern Time lastTimestamp;
 
   body[0] = START_FLAG;
   body[1] = event_type;
-  body[2] = num_datum+3;
-    
+  body[2] = num_datum+HEADER_SIZE;
+  body[3] = lastTimestamp;    
   if(num_datum>0)
-    body[3] = data1;
+    body[HEADER_SIZE] = data1;
   if(num_datum>1)
-    body[4] = data2;
+    body[HEADER_SIZE+1] = data2;
   if(num_datum>2)
-    body[5] = data3;
+    body[HEADER_SIZE+2] = data3;
   if(num_datum>3)
-    body[6] = data4;
+    body[HEADER_SIZE+3] = data4;
   if(num_datum>4)
-    body[7] = data5;
+    body[HEADER_SIZE+4] = data5;
   if(num_datum>5)
-    body[8] = data6;
+    body[HEADER_SIZE+5] = data6;
   if(num_datum>6)
-    body[9] = data7;
-  PositiveWrite(module,body,(num_datum+3)*sizeof(unsigned long));
+    body[HEADER_SIZE+6] = data7;
+  PositiveWrite(module,body,(num_datum+4)*sizeof(body[0]));
 }
 
 void SendConfig(int module, unsigned long event_type, FvwmWindow *t)
 {
   unsigned long body[MAX_BODY_SIZE+HEADER_SIZE];
+  extern Time lastTimestamp;
 
   body[0] = START_FLAG;
   body[1] = event_type;
-  body[2] = 27;
-  body[3] = t->w;
-  body[4] = t->frame;
-  body[5] = (unsigned long)t;
-  body[6] = t->frame_x;
-  body[7] = t->frame_y;
-  body[8] = t->frame_width;
-  body[9] = t->frame_height;
-  body[10] = t->Desk;
-  body[11] = t->flags;
-  body[12] = t->title_height;
-  body[13] = t->boundary_width;
-  body[14] = t->hints.base_width;
-  body[15] = t->hints.base_height;
-  body[16] = t->hints.width_inc;
-  body[17] = t->hints.height_inc;
-  body[18] = t->hints.min_width;
-  body[19] = t->hints.min_height;
-  body[20] = t->hints.max_width;
-  body[21] = t->hints.max_height;
-  body[22] = t->icon_w;
-  body[23] = t->icon_pixmap_w;
-  body[24] = t->hints.win_gravity;
-  body[25] = t->TextPixel;
-  body[26] = t->BackPixel;
+  body[2] = HEADER_SIZE+24;
+  body[3] = lastTimestamp;
+  body[HEADER_SIZE] = t->w;
+  body[HEADER_SIZE+1] = t->frame;
+  body[HEADER_SIZE+2] = (unsigned long)t;
+  body[HEADER_SIZE+3] = t->frame_x;
+  body[HEADER_SIZE+4] = t->frame_y;
+  body[HEADER_SIZE+5] = t->frame_width;
+  body[HEADER_SIZE+6] = t->frame_height;
+  body[HEADER_SIZE+7] = t->Desk;
+  body[HEADER_SIZE+8] = t->flags;
+  body[HEADER_SIZE+9] = t->title_height;
+  body[HEADER_SIZE+10] = t->boundary_width;
+  body[HEADER_SIZE+11] = t->hints.base_width;
+  body[HEADER_SIZE+12] = t->hints.base_height;
+  body[HEADER_SIZE+13] = t->hints.width_inc;
+  body[HEADER_SIZE+14] = t->hints.height_inc;
+  body[HEADER_SIZE+15] = t->hints.min_width;
+  body[HEADER_SIZE+16] = t->hints.min_height;
+  body[HEADER_SIZE+17] = t->hints.max_width;
+  body[HEADER_SIZE+18] = t->hints.max_height;
+  body[HEADER_SIZE+19] = t->icon_w;
+  body[HEADER_SIZE+20] = t->icon_pixmap_w;
+  body[HEADER_SIZE+21] = t->hints.win_gravity;
+  body[HEADER_SIZE+22] = t->TextPixel;
+  body[HEADER_SIZE+23] = t->BackPixel;
   
-  PositiveWrite(module,body,27*sizeof(unsigned long));
+  PositiveWrite(module,body,(HEADER_SIZE+24)*sizeof(body[0]));
 }
 
 
@@ -505,38 +486,40 @@ void BroadcastConfig(unsigned long event_type, FvwmWindow *t)
 {
   unsigned long body[MAX_BODY_SIZE+HEADER_SIZE];
   int i;
+  extern Time lastTimestamp;
 
   body[0] = START_FLAG;
   body[1] = event_type;
-  body[2] = 27;
-  body[3] = t->w;
-  body[4] = t->frame;
-  body[5] = (unsigned long)t;
-  body[6] = t->frame_x;
-  body[7] = t->frame_y;
-  body[8] = t->frame_width;
-  body[9] = t->frame_height;
-  body[10] = t->Desk;
-  body[11] = t->flags;
-  body[12] = t->title_height;
-  body[13] = t->boundary_width;
-  body[14] = t->hints.base_width;
-  body[15] = t->hints.base_height;
-  body[16] = t->hints.width_inc;
-  body[17] = t->hints.height_inc;
-  body[18] = t->hints.min_width;
-  body[19] = t->hints.min_height;
-  body[20] = t->hints.max_width;
-  body[21] = t->hints.max_height;
-  body[22] = t->icon_w;
-  body[23] = t->icon_pixmap_w;
-  body[24] = t->hints.win_gravity;
-  body[25] = t->TextPixel;
-  body[26] = t->BackPixel;
+  body[2] = HEADER_SIZE+24;
+  body[3] = lastTimestamp;
+  body[HEADER_SIZE] = t->w;
+  body[HEADER_SIZE+1] = t->frame;
+  body[HEADER_SIZE+2] = (unsigned long)t;
+  body[HEADER_SIZE+3] = t->frame_x;
+  body[HEADER_SIZE+4] = t->frame_y;
+  body[HEADER_SIZE+5] = t->frame_width;
+  body[HEADER_SIZE+6] = t->frame_height;
+  body[HEADER_SIZE+7] = t->Desk;
+  body[HEADER_SIZE+8] = t->flags;
+  body[HEADER_SIZE+9] = t->title_height;
+  body[HEADER_SIZE+10] = t->boundary_width;
+  body[HEADER_SIZE+11] = t->hints.base_width;
+  body[HEADER_SIZE+12] = t->hints.base_height;
+  body[HEADER_SIZE+13] = t->hints.width_inc;
+  body[HEADER_SIZE+14] = t->hints.height_inc;
+  body[HEADER_SIZE+15] = t->hints.min_width;
+  body[HEADER_SIZE+16] = t->hints.min_height;
+  body[HEADER_SIZE+17] = t->hints.max_width;
+  body[HEADER_SIZE+18] = t->hints.max_height;
+  body[HEADER_SIZE+19] = t->icon_w;
+  body[HEADER_SIZE+20] = t->icon_pixmap_w;
+  body[HEADER_SIZE+21] = t->hints.win_gravity;
+  body[HEADER_SIZE+22] = t->TextPixel;
+  body[HEADER_SIZE+23] = t->BackPixel;
 
   for(i=0;i<npipes;i++)   
     {  
-      PositiveWrite(i,body,27*sizeof(unsigned long));
+      PositiveWrite(i,body,(HEADER_SIZE+24)*sizeof(body[0]));
     }
 }
 
@@ -545,25 +528,27 @@ void BroadcastName(unsigned long event_type, unsigned long data1,
 {
   int l,i;
   unsigned long *body;
+  extern Time lastTimestamp;
 
 
   if(name==NULL)
     return;
-  l=strlen(name)/(sizeof(unsigned long))+7;
-  body = (unsigned long *)safemalloc(l*sizeof(unsigned long));
+  l=strlen(name)/(sizeof(*body))+HEADER_SIZE+4;
+  body = (unsigned long *)safemalloc(l*sizeof(*body));
 
   body[0] = START_FLAG;
   body[1] = event_type;
   body[2] = l;
+  body[3] = lastTimestamp;
 
-  body[3] = data1;
-  body[4] = data2;
-  body[5] = data3; 
-  strcpy((char *)&body[6],name);
+  body[HEADER_SIZE] = data1;
+  body[HEADER_SIZE+1] = data2;
+  body[HEADER_SIZE+2] = data3; 
+  strcpy((char *)&body[HEADER_SIZE+3],name);
 
 
   for(i=0;i<npipes;i++)   
-    PositiveWrite(i,(unsigned long *)body, l*sizeof(unsigned long));
+    PositiveWrite(i,(unsigned long *)body, l*sizeof(*body));
       
   free(body);
 
@@ -576,20 +561,22 @@ void SendName(int module, unsigned long event_type,
 {
   int l;
   unsigned long *body;
+  extern Time lastTimestamp;
 
   if(name == NULL)
     return;
-  l=strlen(name)/(sizeof(unsigned long))+7;
+  l=strlen(name)/(sizeof(unsigned long))+HEADER_SIZE+4;
   body = (unsigned long *)safemalloc(l*sizeof(unsigned long));
 
   body[0] = START_FLAG;
   body[1] = event_type;
   body[2] = l;
+  body[3] = lastTimestamp;
 
-  body[3] = data1;
-  body[4] = data2;
-  body[5] = data3; 
-  strcpy((char *)&body[6],name);
+  body[HEADER_SIZE] = data1;
+  body[HEADER_SIZE+1] = data2;
+  body[HEADER_SIZE+2] = data3; 
+  strcpy((char *)&body[HEADER_SIZE+3],name);
 
   PositiveWrite(module,(unsigned long *)body, l*sizeof(unsigned long));
 
@@ -683,3 +670,73 @@ void FlushQueue(int module)
       DeleteQueueBuff(module);
     }
 }
+
+
+void send_list_func(XEvent *eventp,Window w,FvwmWindow *tmp_win,
+			unsigned long context, char *action,
+			int *Module)
+{
+  FvwmWindow *t;
+
+  if(*Module >= 0)
+    {
+      SendPacket(*Module,M_NEW_DESK,1,Scr.CurrentDesk,0,0,0,0,0,0);
+      SendPacket(*Module,M_NEW_PAGE,5,Scr.Vx,Scr.Vy,Scr.CurrentDesk,
+		 Scr.VxMax,Scr.VyMax,0,0);
+      if(Scr.Hilite != NULL)
+	SendPacket(*Module,M_FOCUS_CHANGE,5,Scr.Hilite->w,Scr.Hilite->frame,
+		   (unsigned long)Scr.Hilite,Scr.HiColors.fore,
+		   Scr.HiColors.back,0,0);
+      else
+	SendPacket(*Module,M_FOCUS_CHANGE,5,0,0,0,Scr.HiColors.fore,
+		   Scr.HiColors.back,0,0);
+      if (Scr.DefaultIcon != NULL)
+	SendName(*Module,M_DEFAULTICON,0,0,0,Scr.DefaultIcon);
+      for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	{
+	  SendConfig(*Module,M_CONFIGURE_WINDOW,t);
+	  SendName(*Module,M_WINDOW_NAME,t->w,t->frame,
+		   (unsigned long)t,t->name);
+	  SendName(*Module,M_ICON_NAME,t->w,t->frame,
+		   (unsigned long)t,t->icon_name);
+         if (t->icon_bitmap_file != NULL &&
+	     t->icon_bitmap_file != Scr.DefaultIcon) 
+           SendName(*Module,M_ICON_FILE,t->w,t->frame,
+                    (unsigned long)t,t->icon_bitmap_file);
+	  SendName(*Module,M_RES_CLASS,t->w,t->frame,
+		   (unsigned long)t,t->class.res_class);
+	  SendName(*Module,M_RES_NAME,t->w,t->frame,
+		   (unsigned long)t,t->class.res_name);
+	  
+	  if((t->flags & ICONIFIED)&&(!(t->flags & ICON_UNMAPPED)))
+	    SendPacket(*Module,M_ICONIFY,7,t->w,t->frame,
+		       (unsigned long)t,
+		       t->icon_x_loc,t->icon_y_loc,
+		       t->icon_w_width, 
+		       t->icon_w_height+t->icon_p_height);
+	  if((t->flags & ICONIFIED) && (t->flags & ICON_UNMAPPED))
+	    SendPacket(*Module,M_ICONIFY,7,t->w,t->frame,
+		       (unsigned long)t,0,0,0,0);
+	}
+      if(Scr.Hilite == NULL)
+	{
+	  Broadcast(M_FOCUS_CHANGE,5,0,0,0,Scr.HiColors.fore,
+		    Scr.HiColors.back,0,0);
+	}
+      else 
+	{
+	  Broadcast(M_FOCUS_CHANGE,5,Scr.Hilite->w,
+		    Scr.Hilite->frame,(unsigned long)Scr.Hilite,
+		    Scr.HiColors.fore,Scr.HiColors.back,0,0);
+	}
+      SendPacket(*Module,M_END_WINDOWLIST,0,0,0,0,0,0,0,0);
+    }
+}
+void set_mask_function(XEvent *eventp,Window w,FvwmWindow *tmp_win,
+		     unsigned long context, char *action,int* Module)
+{
+  int n, val1_unit;
+  long val1;
+  n = GetOneArgument(action, &val1, &val1_unit);
+  PipeMask[*Module] = (unsigned long)val1;
+}
