@@ -39,8 +39,39 @@
 #define F_POPUP			19	/* string */
 #define F_WAIT                  20
 #define F_CLOSE                 21
-#define F_QUICKIE               22
-
+#define F_SET_MASK              23
+#define F_ADDMENU               24
+#define F_ADDFUNC               25
+#define F_STYLE                 26
+#define F_EDGE_SCROLL           27
+#define F_PIXMAP_PATH           28
+#define F_ICON_PATH             29
+#define F_MODULE_PATH           30
+#define F_HICOLOR               31
+#define F_SETDESK               32
+#define F_MOUSE                 34
+#define F_KEY                   35
+#define F_OPAQUE                36
+#define F_XOR                   37
+#define F_CLICK                 38
+#define F_MENUSTYLE             39
+#define F_ICONFONT              40
+#define F_WINDOWFONT            41
+#define F_EDGE_RES              42
+#define F_BUTTON_STYLE          43
+#define F_READ                  44
+#define F_ADDMENU2              45
+#define F_NEXT                  46
+#define F_PREV                  47
+#define F_NONE                  48
+#define F_STAYSUP	        49 /* string */
+#define F_RECAPTURE             50
+#define F_CONFIG_LIST	        51
+#define F_DESTROY_MENU	        52
+#define F_ZAP	                53
+#define F_QUIT_SCREEN		54
+#define F_COLORMAP_FOCUS        55
+#define F_TITLESTYLE            56
 /* Functions which require a target window */
 #define F_RESIZE		100
 #define F_RAISE			101
@@ -54,7 +85,6 @@
 #define F_MAXIMIZE              109
 #define F_FOCUS                 110
 
-
 /* Functions for use by modules only! */
 #define F_SEND_WINDOW_LIST     1000
 
