@@ -69,8 +69,10 @@
 #define BOTTOM_RIGHT 16
 #define MAX_CURSORS 18
 
-/* Maximum number of icon boxes that are allowed */
-#define MAX_BOXES 4
+/* colormap focus styes */
+#define COLORMAP_FOLLOWS_MOUSE 1 /* default */
+#define COLORMAP_FOLLOWS_FOCUS 2
+
 
 #ifndef NON_VIRTUAL
 typedef struct 
@@ -96,7 +98,6 @@ typedef struct ScreenInfo
 				 * windows have it */
 #ifndef NON_VIRTUAL
   PanFrame PanFrameTop,PanFrameLeft,PanFrameRight,PanFrameBottom;
-  int usePanFrames;		/* toggle to disable them */
 #endif  
 
   Pixmap gray_bitmap;           /*dark gray pattern for shaded out menu items*/
@@ -104,18 +105,13 @@ typedef struct ScreenInfo
   Pixmap light_gray_pixmap;     /* light gray pattern for inactive borders */
   Pixmap sticky_gray_pixmap;     /* light gray pattern for sticky borders */
 
-  MouseButton *MouseButtonRoot;
-  FuncKey FuncKeyRoot;
+  Binding *AllBindings;
+  MenuRoot *AllMenus;
 
   int root_pushes;		/* current push level to install root
 				   colormap windows */
   FvwmWindow *pushed_window;	/* saved window to install when pushes drops
 				   to zero */
-#ifndef NO_PAGER
-  FvwmWindow *FvwmPager;
-  Window Pager_w;
-  Window CPagerWin;
-#endif
   Cursor FvwmCursors[MAX_CURSORS];
 
   name_list *TheList;		/* list of window names with attributes */
@@ -126,20 +122,13 @@ typedef struct ScreenInfo
   ColorPair MenuRelief;
 
   ColorPair StdColors; 	/* standard fore/back colors */
-  ColorPair StickyColors; 	/* sticky fore/back colors */
-  ColorPair StickyRelief; 	/* sticky hilight colors */
   ColorPair HiColors; 	/* standard fore/back colors */
   ColorPair StdRelief;
   ColorPair HiRelief;
   MyFont StdFont;     	/* font structure */
   MyFont WindowFont;   	/* font structure for window titles */
-#ifndef NO_PAGER
-  MyFont PagerFont;   	/* font struct for window labels in pager (optional)*/
-#endif
   MyFont IconFont;      /* for icon labels */
   
-  GC NormalGC;		        /* normal GC for menus, pager, resize window */
-  GC StippleGC;		        /* normal GC for menus, pager, resize window */
   GC DrawGC;			/* GC to draw lines for move and resize */
   GC HiReliefGC;                /* GC for highlighted window relief */
   GC HiShadowGC;                /* GC for highlighted window shadow */
@@ -147,25 +136,27 @@ typedef struct ScreenInfo
   GC MenuStippleGC;
   GC MenuReliefGC;
   GC MenuShadowGC;
-  GC StdReliefGC;               /* GC for unselected window relief */
-  GC StdShadowGC;               /* GC for unselected window shadow */
   GC ScratchGC1;
   GC ScratchGC2;
-
-  GC StickyReliefGC;               /* GC for unselected sticky window relief */
-  GC StickyShadowGC;               /* GC for unselected sticky window shadow */
-  GC FontGC;                    /* GC for non-standard fonts */
-
+  GC ScratchGC3;
   int SizeStringWidth;	        /* minimum width of size window */
   int CornerWidth;	        /* corner width for decoratedwindows */
   int BoundaryWidth;	        /* frame width for decorated windows */
   int NoBoundaryWidth;	        /* frame width for decorated windows */
   int TitleHeight;		/* height of the title bar window */
+  struct TitleStyle             /* how do the titles look? */
+  {
+    int justify;                /* left, right, centered */
+    int appearance;             /* raised, sunk, flat, not implemented yet */
+  } TitleStyle;
   FvwmWindow *Hilite;		/* the fvwm window that is highlighted 
 				 * except for networking delays, this is the
 				 * window which REALLY has the focus */
   FvwmWindow *Focus;            /* Last window which Fvwm gave the focus to 
                                  * NOT the window that really has the focus */
+  Window UnknownWinFocused;      /* None, if the focus is nowhere or on an fvwm
+				 * managed window. Set to id of otherwindow 
+				 * with focus otherwise */
   FvwmWindow *Ungrabbed;
   FvwmWindow *PreviousFocus;    /* Window which had focus before fvwm stole it
 				 * to do moves/menus/etc. */
@@ -174,11 +165,9 @@ typedef struct ScreenInfo
   int EdgeScrollY;              /* #pixels to scroll on screen edge */
   unsigned char buttons2grab;   /* buttons to grab in click to focus mode */
   unsigned long flags;
-  int IconBoxes[MAX_BOXES][4];
   int NumBoxes;
   int randomx;                  /* values used for randomPlacement */
   int randomy;
-  unsigned VScale;              /* Panner scale factor */
   FvwmWindow *LastWindowRaised; /* Last window which was raised. Used for raise
 				 * lower func. */
   int VxMax;                    /* Max location for top left of virt desk*/
@@ -193,13 +182,11 @@ typedef struct ScreenInfo
   int right_button_styles[2][5];
 
   int ClickTime;               /*Max button-click delay for Function built-in*/
-  int AutoRaiseDelay;          /* Delay between setting focus and raising win*/
   int ScrollResistance;        /* resistance to scrolling in desktop */
   int MoveResistance;          /* res to moving windows over viewport edge */
   int OpaqueSize;
   int CurrentDesk;             /* The current desktop number */
-  MenuRoot *InitFunction;
-  MenuRoot *RestartFunction;
+  int ColormapFocus;           /* colormap focus style */
 
   /* new style */
   Boolean use_new_style[10];   /* if true then use custom shapes */
@@ -216,32 +203,18 @@ typedef struct ScreenInfo
 
 extern ScreenInfo Scr;
 
+/* for the titlestyle values: */
+#define JUSTIFY_CENTER 0
+#define JUSTIFY_LEFT   1
+#define JUSTIFY_RIGHT  2
+#define TITLE_RAISED 0
+#define TITLE_SUNK   1
+#define TITLE_FLAT   2
+#define TITLE_MWM    3
+
 /* for the flags value - these used to be seperate Bool's */
-#define ClickToFocus               (1) /* Focus follows mouse, or click to focus?*/
-#define DecorateTransients         (2) /* decorate transient windows? */
-#define DontMoveOff                (4) /* make sure all windows stay on desktop*/
-#define RandomPlacement            (8) /* place windows in random locations? */
-#define SuppressIcons             (16) /* prevent generation of icon windows */
-#define StickyIcons               (32) /* Icons always sticky? */
+#define WindowsCaptured            (1)
 #define EdgeWrapX                 (64) /* Should EdgeScroll wrap around? */
 #define EdgeWrapY                (128) 
-#define CenterOnCirculate        (256) /* center window when circulating? */
-#define MWMBorders               (512)
 #define MWMMenus                (1024)
-#define MWMButtons              (2048)
-#define MWMDecorHints           (4096)
-#define NoPPosition             (8192)
-#define SMART_PLACEMENT        (16384)
-#define CirculateSkipIcons     (32768)
-#define StubbornIcons          (65536)
-#define StubbornPlacement     (131072)
-#define StubbornIconPlacement (262144)
-#define OpaqueResize          (524288)
-#define MWMFunctionHints     (1048576)
-#define MWMHintOverride      (2097152)
-#define BackingStore         (4194304)
-#define AppsBackingStore     (8388608)
-#define SaveUnders          (16777216)
-#define Lenience            (33554432)
-#define SloppyFocus         (67108864)
 #endif /* _SCREEN_ */
